@@ -16,6 +16,9 @@
 // https://github.com/hrydgard/ppsspp and http://www.ppsspp.org/.
 
 #include "ppsspp_config.h"
+
+#include <cctype>
+#include <cmath>
 #include <algorithm>
 
 #include "Common/Data/Text/I18n.h"
@@ -28,122 +31,36 @@
 #include "Core/Dialog/PSPOskDialog.h"
 #include "Core/Util/PPGeDraw.h"
 #include "Core/HLE/sceCtrl.h"
+#include "Core/HLE/ErrorCodes.h"
 #include "Core/HLE/sceUtility.h"
 #include "Core/HW/Display.h"
 #include "Core/Config.h"
 #include "Core/Reporting.h"
-#include "GPU/GPUState.h"
-
-#ifndef _WIN32
-#include <ctype.h>
-#include <math.h>
-#endif
 
 // These are rough, it seems to take a long time to init, and probably depends on threads.
 // TODO: This takes like 700ms on a PSP but that's annoyingly long.
 const static int OSK_INIT_DELAY_US = 300000;
 const static int OSK_SHUTDOWN_DELAY_US = 40000;
 
-static std::map<std::string, std::pair<std::string, int>, std::less<>> languageMapping;
-
 const uint8_t numKeyCols[OSK_KEYBOARD_COUNT] = {12, 12, 13, 13, 12, 12, 12, 12, 12};
 const uint8_t numKeyRows[OSK_KEYBOARD_COUNT] = {4, 4, 6, 6, 5, 4, 4, 4, 4};
 
-// Japanese (Kana) diacritics
-static const wchar_t diacritics[2][103] =
-{
-	{L"かがきぎくぐけげこごさざしじすずせぜそぞただちぢつづてでとどはばぱばひびぴびふぶぷぶへべぺべほぼぽぼウヴカガキギクグケゲコゴサザシジスズセゼソゾタダチヂツヅテデトドハバパバヒビピビフブプブヘベペベホボポボ"},
-	{L"はぱばぱひぴびぴふぷぶぷへぺべぺほぽぼぽハパバパヒピビピフプブプヘペベペホポボポ"}
-};
-
-// Korean (Hangul) consonant
-static const wchar_t kor_cons[] = L"ㄱㄲㄴㄷㄸㄹㅁㅂㅃㅅㅆㅇㅈㅉㅊㅋㅌㅍㅎ";
-
-// Korean (Hangul) vowels, Some vowels are not used, they will be spaces
-static const wchar_t kor_vowel[] = L"ㅏㅐㅑㅒㅓㅔㅕㅖㅗ   ㅛㅜ   ㅠㅡ ㅣ";
-
 // Korean (Hangul) vowel Combination key
-const uint8_t kor_vowelCom[] = {0,8,9,1,8,10,20,8,11,4,13,14,5,13,15,20,13,16,20,18,19};
-
-// Korean (Hangul) last consonant(diacritics)
-static const wchar_t kor_lcons[] = L"ㄱㄲㄳㄴㄵㄶㄷㄹㄺㄻㄼㄽㄾㄿㅀㅁㅂㅄㅅㅆㅇㅈㅊㅋㅌㅍㅎ";
+static const uint8_t kor_vowelCom[21] = { 0,8,9,1,8,10,20,8,11,4,13,14,5,13,15,20,13,16,20,18,19 };
 
 // Korean (Hangul) last consonant Combination key
-const uint8_t kor_lconsCom[] = {18,0,2,21,3,4,26,3,5,0,7,8,15,7,9,16,7,10,18,7,11,24,7,12,25,7,13,26,7,14,18,16,17};
+static const uint8_t kor_lconsCom[33] = { 18,0,2,21,3,4,26,3,5,0,7,8,15,7,9,16,7,10,18,7,11,24,7,12,25,7,13,26,7,14,18,16,17 };
 
 // Korean (Hangul) last consonant Separation key
-const uint8_t kor_lconsSpr[] = {2,1,9,4,4,12,5,4,18,8,8,0,9,8,6,10,8,7,11,8,9,12,8,16,13,8,17,14,8,18,17,17,9};
+static const uint8_t kor_lconsSpr[33] = { 2,1,9,4,4,12,5,4,18,8,8,0,9,8,6,10,8,7,11,8,9,12,8,16,13,8,17,14,8,18,17,17,9 };
 
-static const char16_t oskKeys[OSK_KEYBOARD_COUNT][6][14] =
+static const std::string_view OskKeyboardNames[] =
 {
-	{
-		// Latin Lowercase
-		{u"1234567890-+"},
-		{u"qwertyuiop[]"},
-		{u"asdfghjkl;@~"},
-		{u"zxcvbnm,./?\\"},
-	},
-	{
-		// Latin Uppercase
-		{u"!@#$%^&*()_+"},
-		{u"QWERTYUIOP{}"},
-		{u"ASDFGHJKL:\"`"},
-		{u"ZXCVBNM<>/?|"},
-	},
-	{
-		// Hiragana
-		{u"あかさたなはまやらわぁゃっ"},
-		{u"いきしちにひみ　り　ぃ　　"},
-		{u"うくすつぬふむゆるをぅゅ゛"},
-		{u"えけせてねへめ　れ　ぇ　゜"},
-		{u"おこそとのほもよろんぉょー"},
-		{u"・。、「」『』〜     "},
-	},
-	{
-		// Katakana
-		{u"アカサタナハマヤラワァャッ"},
-		{u"イキシチニヒミ　リ　ィ　　"},
-		{u"ウクスツヌフムユルヲゥュ゛"},
-		{u"エケセテネヘメ　レ　ェ　゜"},
-		{u"オコソトノホモヨロンォョー"},
-		{u"・。、「」『』〜     "},
-	},
-	{
-		// Korean(Hangul)
-		{u"1234567890-+"},
-		{u"ㅃㅉㄸㄲㅆ!@#$%^&"},
-		{u"ㅂㅈㄷㄱㅅㅛㅕㅑㅐㅔ[]"},
-		{u"ㅁㄴㅇㄹㅎㅗㅓㅏㅣ;@~"},
-		{u"ㅋㅌㅊㅍㅠㅜㅡ<>/?|"},
-	},
-	{
-		// Russian Lowercase
-		{u"1234567890-+"},
-		{u"йцукенгшщзхъ"},
-		{u"фывапролджэё"},
-		{u"ячсмитьбю/?|"},
-	},
-	{
-		// Russian Uppercase
-		{u"!@#$%^&*()_+"},
-		{u"ЙЦУКЕНГШЩЗХЪ"},
-		{u"ФЫВАПРОЛДЖЭЁ"},
-		{u"ЯЧСМИТЬБЮ/?|"},
-	},
-	{
-		// Latin Full-width Lowercase
-		{ u"１２３４５６７８９０－＋" },
-		{ u"ｑｗｅｒｔｙｕｉｏｐ［］" },
-		{ u"ａｓｄｆｇｈｊｋｌ；＠～" },
-		{ u"ｚｘｃｖｂｎｍ，．／？￥￥" },
-	},
-	{
-		// Latin Full-width Uppercase
-		{ u"！＠＃＄％＾＆＊（）＿＋" },
-		{ u"ＱＷＥＲＴＹＵＩＯＰ｛｝" },
-		{ u"ＡＳＤＦＧＨＪＫＬ：￥”‘" },
-		{ u"ＺＸＣＶＢＮＭ＜＞／？｜" },
-	},
+	"en_US",
+	"ja_JP",
+	"ko_KR",
+	"ru_RU",
+	"English Full-width",
 };
 
 // This isn't a complete representation of these flags, it just helps ensure we show the right keyboards.
@@ -191,7 +108,7 @@ void PSPOskDialog::ConvertUCS2ToUTF8(std::string& _string, const PSPPointer<u16_
 	char stringBuffer[maxLength + 1];
 	char *string = stringBuffer;
 
-	u16_le *input = &em_address[0];
+	const u16_le *input = &em_address[0];
 	int c;
 	while ((c = *input++) != 0 && string < stringBuffer + maxLength)
 	{
@@ -222,7 +139,7 @@ void GetWideStringFromPSPPointer(std::u16string& _string, const PSPPointer<u16_l
 	char16_t stringBuffer[maxLength + 1];
 	char16_t *string = stringBuffer;
 
-	u16_le *input = &em_address[0];
+	const u16_le *input = &em_address[0];
 	int c;
 	while ((c = *input++) != 0 && string < stringBuffer + maxLength)
 		*string++ = c;
@@ -298,32 +215,32 @@ static bool IsKeyboardShiftValid(s32 inputType, OskKeyboardLanguage lang, OskKey
 int PSPOskDialog::Init(u32 oskPtr) {
 	// Ignore if already running
 	if (GetStatus() != SCE_UTILITY_STATUS_NONE) {
-		ERROR_LOG_REPORT(SCEUTILITY, "sceUtilityOskInitStart: invalid status");
+		ERROR_LOG_REPORT(Log::sceUtility, "sceUtilityOskInitStart: invalid status");
 		return SCE_ERROR_UTILITY_INVALID_STATUS;
 	}
 	// Seems like this should crash?
 	if (!Memory::IsValidAddress(oskPtr)) {
-		ERROR_LOG_REPORT(SCEUTILITY, "sceUtilityOskInitStart: invalid params (%08x)", oskPtr);
+		ERROR_LOG_REPORT(Log::sceUtility, "sceUtilityOskInitStart: invalid params (%08x)", oskPtr);
 		return -1;
 	}
 
 	oskParams = oskPtr;
 	if (oskParams->base.size != sizeof(SceUtilityOskParams))
 	{
-		ERROR_LOG_REPORT(SCEUTILITY, "sceUtilityOskInitStart: invalid size %d", oskParams->base.size);
+		ERROR_LOG_REPORT(Log::sceUtility, "sceUtilityOskInitStart: invalid size %d", oskParams->base.size);
 		return SCE_ERROR_UTILITY_INVALID_PARAM_SIZE;
 	}
 	// Also seems to crash.
 	if (!oskParams->fields.IsValid())
 	{
-		ERROR_LOG_REPORT(SCEUTILITY, "sceUtilityOskInitStart: invalid field data (%08x)", oskParams->fields.ptr);
+		ERROR_LOG_REPORT(Log::sceUtility, "sceUtilityOskInitStart: invalid field data (%08x)", oskParams->fields.ptr);
 		return -1;
 	}
 
 	if (oskParams->unk_60 != 0)
-		WARN_LOG_REPORT(SCEUTILITY, "sceUtilityOskInitStart: unknown param is non-zero (%08x)", oskParams->unk_60);
+		WARN_LOG_REPORT(Log::sceUtility, "sceUtilityOskInitStart: unknown param is non-zero (%08x)", oskParams->unk_60);
 	if (oskParams->fieldCount != 1)
-		WARN_LOG_REPORT(SCEUTILITY, "sceUtilityOskInitStart: unsupported field count %d", oskParams->fieldCount);
+		WARN_LOG_REPORT(Log::sceUtility, "sceUtilityOskInitStart: unsupported field count %d", oskParams->fieldCount);
 
 	ChangeStatusInit(OSK_INIT_DELAY_US);
 	selectedChar = 0;
@@ -346,8 +263,6 @@ int PSPOskDialog::Init(u32 oskPtr) {
 			inputChars += c;
 	}
 
-	languageMapping = g_Config.GetLangValuesMapping();
-
 	// Eat any keys pressed before the dialog inited.
 	UpdateButtons();
 	InitCommon();
@@ -369,12 +284,12 @@ std::u16string PSPOskDialog::CombinationKorean(bool isInput)
 	int selectedCol = selectedChar % numKeyCols[currentKeyboard];
 
 	if (inputChars.size() == 0) {
-		wchar_t sw = oskKeys[currentKeyboard][selectedRow][selectedCol];
+		wchar_t sw = OskKeyAt(currentKeyboard, selectedRow, selectedCol);
 
 		if (inputChars.size() < FieldMaxLength()) {
 			string += sw;
 
-			i_value[0] = GetIndex(kor_cons, sw);
+			i_value[0] = GetIndex(KorCons(), sw);
 
 			if(i_value[0] != -1 && isInput == true)
 				i_level = 1;
@@ -384,14 +299,14 @@ std::u16string PSPOskDialog::CombinationKorean(bool isInput)
 	} else {
 		for(u32 i = 0; i < inputChars.size(); i++) {
 			if(i + 1 == inputChars.size()) {
-				wchar_t sw = oskKeys[currentKeyboard][selectedRow][selectedCol];
+				wchar_t sw = OskKeyAt(currentKeyboard, selectedRow, selectedCol);
 
 				if(i_level == 0) {
 					string += inputChars[i];
 					if (inputChars.size() < FieldMaxLength()) {
 						string += sw;
 
-						i_value[0] = GetIndex(kor_cons, sw);
+						i_value[0] = GetIndex(KorCons(), sw);
 
 						if(i_value[0] != -1 && isInput == true)
 							i_level = 1;
@@ -399,7 +314,7 @@ std::u16string PSPOskDialog::CombinationKorean(bool isInput)
 						isCombinated = false;
 					}
 				} else if(i_level == 1) {
-					i_value[1] = GetIndex(kor_vowel, sw);
+					i_value[1] = GetIndex(KorVowel(), sw);
 
 					if(i_value[1] == -1) {
 						string += inputChars[i];
@@ -407,7 +322,7 @@ std::u16string PSPOskDialog::CombinationKorean(bool isInput)
 							string += sw;
 
 							if(isInput == true) {
-								i_value[0] = GetIndex(kor_cons, sw);
+								i_value[0] = GetIndex(KorCons(), sw);
 
 								if(i_value[0] != -1)
 									i_level = 1;
@@ -426,7 +341,7 @@ std::u16string PSPOskDialog::CombinationKorean(bool isInput)
 						}
 					}
 				} else if(i_level == 2) {
-					int tmp = GetIndex(kor_vowel, sw);
+					int tmp = GetIndex(KorVowel(), sw);
 					if(tmp != -1) {
 						int tmp2 = -1;
 						for(size_t j = 0; j < sizeof(kor_vowelCom) / 4; j+=3) {
@@ -456,7 +371,7 @@ std::u16string PSPOskDialog::CombinationKorean(bool isInput)
 							}
 						}
 					} else {
-						int tmp2 = GetIndex(kor_lcons, sw);
+						int tmp2 = GetIndex(KorLCons(), sw);
 
 						if (tmp2 == -1) {
 							string += inputChars[i];
@@ -464,7 +379,7 @@ std::u16string PSPOskDialog::CombinationKorean(bool isInput)
 								string += sw;
 
 								if (isInput == true) {
-									i_value[0] = GetIndex(kor_cons, sw);
+									i_value[0] = GetIndex(KorCons(), sw);
 
 									if(i_value[0] != -1)
 										i_level = 1;
@@ -486,7 +401,7 @@ std::u16string PSPOskDialog::CombinationKorean(bool isInput)
 						}
 					}
 				} else if(i_level == 3) {
-					int tmp = GetIndex(kor_lcons, sw);
+					int tmp = GetIndex(KorLCons(), sw);
 					if(tmp != -1) {
 						int tmp2 = -1;
 						for(size_t j = 0; j < sizeof(kor_lconsCom) / 4; j+=3) {
@@ -509,7 +424,7 @@ std::u16string PSPOskDialog::CombinationKorean(bool isInput)
 								string += sw;
 
 								if(isInput == true) {
-									i_value[0] = GetIndex(kor_cons, sw);
+									i_value[0] = GetIndex(KorCons(), sw);
 
 									if(i_value[0] != -1)
 										i_level = 1;
@@ -521,14 +436,14 @@ std::u16string PSPOskDialog::CombinationKorean(bool isInput)
 							}
 						}
 					} else {
-						int tmp3 = GetIndex(kor_vowel, sw);
+						int tmp3 = GetIndex(KorVowel(), sw);
 						if (tmp3 == -1) {
 							string += inputChars[i];
 							if (inputChars.size() < FieldMaxLength()) {
 								string += sw;
 
 								if(isInput == true) {
-									i_value[0] = GetIndex(kor_cons, sw);
+									i_value[0] = GetIndex(KorCons(), sw);
 
 									if(i_value[0] != -1)
 										i_level = 1;
@@ -560,7 +475,7 @@ std::u16string PSPOskDialog::CombinationKorean(bool isInput)
 										i_level = 2;
 									}
 								} else {
-									int tmp4 = GetIndex(kor_cons, kor_lcons[i_value[2]]);
+									int tmp4 = GetIndex(KorCons(), KorLCons()[i_value[2]]);
 
 									if (tmp4 != -1) {
 										u16 code = 0xAC00 + i_value[0] * 0x24C + i_value[1] * 0x1C;
@@ -621,17 +536,17 @@ std::u16string PSPOskDialog::CombinationString(bool isInput)
 			i_level = 0;
 		}
 
-		if(oskKeys[currentKeyboard][selectedRow][selectedCol] == L'゛')
+		if(OskKeyAt(currentKeyboard, selectedRow, selectedCol) == L'゛')
 		{
 			for(u32 i = 0; i < inputChars.size(); i++)
 			{
 				if(i + 1 == inputChars.size())
 				{
-					for(u32 j = 0; j < wcslen(diacritics[0]); j+=2)
+					for(u32 j = 0; j < wcslen(JapDiacritics(0)); j+=2)
 					{
-						if(inputChars[i] == diacritics[0][j])
+						if(inputChars[i] == JapDiacritics(0)[j])
 						{
-							string += diacritics[0][j + 1];
+							string += JapDiacritics(0)[j + 1];
 							isCombinated = true;
 							break;
 						}
@@ -648,17 +563,17 @@ std::u16string PSPOskDialog::CombinationString(bool isInput)
 				}
 			}
 		}
-		else if(oskKeys[currentKeyboard][selectedRow][selectedCol] == L'゜')
+		else if(OskKeyAt(currentKeyboard, selectedRow, selectedCol) == L'゜')
 		{
 			for(u32 i = 0; i < inputChars.size(); i++)
 			{
 				if(i + 1 == inputChars.size())
 				{
-					for(u32 j = 0; j < wcslen(diacritics[1]); j+=2)
+					for(u32 j = 0; j < wcslen(JapDiacritics(1)); j+=2)
 					{
-						if(inputChars[i] == diacritics[1][j])
+						if(inputChars[i] == JapDiacritics(1)[j])
 						{
-							string += diacritics[1][j + 1];
+							string += JapDiacritics(1)[j + 1];
 							isCombinated = true;
 							break;
 						}
@@ -684,7 +599,7 @@ std::u16string PSPOskDialog::CombinationString(bool isInput)
 
 			if (string.size() < FieldMaxLength())
 			{
-				string += oskKeys[currentKeyboard][selectedRow][selectedCol];
+				string += OskKeyAt(currentKeyboard, selectedRow, selectedCol);
 			}
 			isCombinated = true;
 		}
@@ -720,7 +635,7 @@ void PSPOskDialog::RemoveKorean()
 		else
 		{
 			i_level = 1;
-			inputChars += kor_cons[i_value[0]];
+			inputChars += KorCons()[i_value[0]];
 		}
 	}
 	else if(i_level == 3)
@@ -795,7 +710,7 @@ void PSPOskDialog::RenderKeyboard()
 	float title = (480.0f - (0.5f * drawLimit)) / 2.0f;
 
 	PPGeStyle descStyle = FadedStyle(PPGeAlign::BOX_CENTER, 0.5f);
-	PPGeDrawText(oskDesc.c_str(), title, 20, descStyle);
+	PPGeDrawText(oskDesc, title, 20, descStyle);
 
 	PPGeStyle textStyle = FadedStyle(PPGeAlign::BOX_HCENTER, 0.5f);
 
@@ -815,7 +730,7 @@ void PSPOskDialog::RenderKeyboard()
 		{
 			temp[0] = result[drawIndex];
 			ConvertUCS2ToUTF8(buffer, temp);
-			PPGeDrawText(buffer.c_str(), previewLeftSide + (i * characterWidth), 40.0f, textStyle);
+			PPGeDrawText(buffer, previewLeftSide + (i * characterWidth), 40.0f, textStyle);
 		}
 		else
 		{
@@ -833,7 +748,7 @@ void PSPOskDialog::RenderKeyboard()
 
 					ConvertUCS2ToUTF8(buffer, temp);
 
-					PPGeDrawText(buffer.c_str(), previewLeftSide + (i * characterWidth), 40.0f, animStyle);
+					PPGeDrawText(buffer, previewLeftSide + (i * characterWidth), 40.0f, animStyle);
 
 					// Also draw the underline for the same reason.
 					PPGeDrawText("_", previewLeftSide + (i * characterWidth), 40.0f, textStyle);
@@ -841,7 +756,7 @@ void PSPOskDialog::RenderKeyboard()
 				else
 				{
 					ConvertUCS2ToUTF8(buffer, temp);
-					PPGeDrawText(buffer.c_str(), previewLeftSide + (i * characterWidth), 40.0f, textStyle);
+					PPGeDrawText(buffer, previewLeftSide + (i * characterWidth), 40.0f, textStyle);
 				}
 			}
 			else
@@ -855,15 +770,15 @@ void PSPOskDialog::RenderKeyboard()
 	{
 		for (int col = 0; col < numKeyCols[currentKeyboard]; ++col)
 		{
-			temp[0] = oskKeys[currentKeyboard][row][col];
+			temp[0] = OskKeyAt(currentKeyboard, row, col);
 
 			ConvertUCS2ToUTF8(buffer, temp);
 
 			if (selectedRow == row && col == selectedCol) {
-				PPGeDrawText(buffer.c_str(), keyboardLeftSide + (25.0f * col) + characterWidth / 2.0, 70.0f + (25.0f * row), selectedKeyStyle);
+				PPGeDrawText(buffer, keyboardLeftSide + (25.0f * col) + characterWidth / 2.0, 70.0f + (25.0f * row), selectedKeyStyle);
 				PPGeDrawText("_", keyboardLeftSide + (25.0f * col) + characterWidth / 2.0, 70.0f + (25.0f * row), keyStyle);
 			} else {
-				PPGeDrawText(buffer.c_str(), keyboardLeftSide + (25.0f * col) + characterWidth / 2.0, 70.0f + (25.0f * row), keyStyle);
+				PPGeDrawText(buffer, keyboardLeftSide + (25.0f * col) + characterWidth / 2.0, 70.0f + (25.0f * row), keyStyle);
 			}
 		}
 	}
@@ -893,11 +808,8 @@ int PSPOskDialog::NativeKeyboard() {
 		std::u16string defaultText;
 		GetWideStringFromPSPPointer(defaultText, oskParams->fields[0].intext);
 
-		if (defaultText.empty())
-			defaultText.assign(u"VALUE");
-
 		// There's already ConvertUCS2ToUTF8 in this file. Should we use that instead of the global ones?
-		System_InputBoxGetString(NON_EPHEMERAL_TOKEN, ::ConvertUCS2ToUTF8(titleText), ::ConvertUCS2ToUTF8(defaultText),
+		System_InputBoxGetString(NON_EPHEMERAL_TOKEN, ::ConvertUCS2ToUTF8(titleText), ::ConvertUCS2ToUTF8(defaultText), false,
 			[&](const std::string &value, int) {
 				// Success callback
 				std::lock_guard<std::mutex> guard(nativeMutex_);
@@ -923,7 +835,7 @@ int PSPOskDialog::NativeKeyboard() {
 
 		u32 maxLength = FieldMaxLength();
 		if (inputChars.length() > maxLength) {
-			ERROR_LOG(SCEUTILITY, "NativeKeyboard: input text too long(%d characters/glyphs max), truncating to game-requested length.", maxLength);
+			ERROR_LOG(Log::sceUtility, "NativeKeyboard: input text too long(%d characters/glyphs max), truncating to game-requested length.", maxLength);
 			inputChars.erase(maxLength, std::string::npos);
 		}
 		ChangeStatus(SCE_UTILITY_STATUS_FINISHED, 0);
@@ -1022,12 +934,15 @@ int PSPOskDialog::Update(int animSpeed) {
 			return (const char *)nullptr;
 		}
 
-		// Now, let's grab the name.
-		const char *countryCode = OskKeyboardNames[lang].c_str();
-		const char *language = languageMapping[countryCode].first.c_str();
+		// Now, let's grab the friendly name from the langvaluesmapping.
+		const std::string countryCode(OskKeyboardNames[lang]);
+
+		const auto &languageMapping = GetLangValuesMapping();
+		const auto iter = languageMapping.find(countryCode);
+		const char *language = iter == languageMapping.end() ? "" : iter->second.first.c_str();
 
 		// It seems like this is a "fake" country code for extra keyboard purposes.
-		if (!strcmp(countryCode, "English Full-width"))
+		if (countryCode == "English Full-width")
 			language = "English Full-width";
 
 		return language;
