@@ -15,17 +15,16 @@
 // Official git repository and contact information can be found at
 // https://github.com/hrydgard/ppsspp and http://www.ppsspp.org/.
 
-#include <cstdio>
-#include <cstring>
 #include <algorithm>
+#include <cstring>
 
 #include "Common/Data/Text/I18n.h"
-#include "Common/File/FileUtil.h"
 #include "Common/System/OSD.h"
 #include "Common/Log.h"
 #include "Common/Swap.h"
 #include "Common/File/FileUtil.h"
 #include "Common/File/DirListing.h"
+#include "Common/StringUtils.h"
 #include "Core/Loaders.h"
 #include "Core/FileSystems/BlockDevices.h"
 #include "libchdr/chd.h"
@@ -37,14 +36,15 @@ extern "C"
 #include "ext/libkirk/kirk_engine.h"
 };
 
-std::mutex NPDRMDemoBlockDevice::mutex_;
-
-BlockDevice *constructBlockDevice(FileLoader *fileLoader) {
+BlockDevice *ConstructBlockDevice(FileLoader *fileLoader, std::string *errorString) {
 	if (!fileLoader->Exists()) {
+		// Shouldn't get here really.
+		*errorString = "File doesn't exist";
 		return nullptr;
 	}
 	if (fileLoader->IsDirectory()) {
-		ERROR_LOG(LOADER, "Can't open directory directly as block device: %s", fileLoader->GetPath().c_str());
+		*errorString = "Can't open directory directly as block device: ";
+		*errorString += fileLoader->GetPath().ToString();
 		return nullptr;
 	}
 
@@ -52,23 +52,36 @@ BlockDevice *constructBlockDevice(FileLoader *fileLoader) {
 	size_t size = fileLoader->ReadAt(0, 1, 8, buffer);
 	if (size != 8) {
 		// Bad or empty file
+		*errorString = "File is empty";
 		return nullptr;
 	}
 
+	BlockDevice *device = nullptr;
+
 	// Check for CISO
 	if (!memcmp(buffer, "CISO", 4)) {
-		return new CISOFileBlockDevice(fileLoader);
+		device = new CISOFileBlockDevice(fileLoader);
 	} else if (!memcmp(buffer, "\x00PBP", 4)) {
 		uint32_t psarOffset = 0;
 		size = fileLoader->ReadAt(0x24, 1, 4, &psarOffset);
 		if (size == 4 && psarOffset < fileLoader->FileSize())
-			return new NPDRMDemoBlockDevice(fileLoader);
+			device = new NPDRMDemoBlockDevice(fileLoader);
 	} else if (!memcmp(buffer, "MComprHD", 8)) {
-		return new CHDFileBlockDevice(fileLoader);
+		device = new CHDFileBlockDevice(fileLoader);
 	}
 
-	// Should be just a regular ISO file. Let's open it as a plain block device and let the other systems take over.
-	return new FileBlockDevice(fileLoader);
+	// No check above passed, should be just a regular ISO file. Let's open it as a plain block device and let the other systems take over.
+	if (!device) {
+		device = new FileBlockDevice(fileLoader);
+	}
+
+	if (!device->IsOK()) {
+		*errorString = device->ErrorString();
+		delete device;
+		return nullptr;
+	}
+
+	return device;
 }
 
 void BlockDevice::NotifyReadError() {
@@ -84,24 +97,22 @@ FileBlockDevice::FileBlockDevice(FileLoader *fileLoader)
 	filesize_ = fileLoader->FileSize();
 }
 
-FileBlockDevice::~FileBlockDevice() {
-}
+FileBlockDevice::~FileBlockDevice() {}
 
 bool FileBlockDevice::ReadBlock(int blockNumber, u8 *outPtr, bool uncached) {
 	FileLoader::Flags flags = uncached ? FileLoader::Flags::HINT_UNCACHED : FileLoader::Flags::NONE;
 	size_t retval = fileLoader_->ReadAt((u64)blockNumber * (u64)GetBlockSize(), 1, 2048, outPtr, flags);
 	if (retval != 2048) {
-		DEBUG_LOG(FILESYS, "Could not read 2048 byte block, at block offset %d. Only got %d bytes", blockNumber, (int)retval);
+		DEBUG_LOG(Log::FileSystem, "Could not read 2048 byte block, at block offset %d. Only got %d bytes", blockNumber, (int)retval);
 		return false;
 	}
-
 	return true;
 }
 
 bool FileBlockDevice::ReadBlocks(u32 minBlock, int count, u8 *outPtr) {
 	size_t retval = fileLoader_->ReadAt((u64)minBlock * (u64)GetBlockSize(), 2048, count, outPtr);
 	if (retval != (size_t)count) {
-		ERROR_LOG(FILESYS, "Could not read %d blocks, at block offset %d. Only got %d blocks", count, minBlock, (int)retval);
+		ERROR_LOG(Log::FileSystem, "Could not read %d blocks, at block offset %d. Only got %d blocks", count, minBlock, (int)retval);
 		return false;
 	}
 	return true;
@@ -145,17 +156,22 @@ CISOFileBlockDevice::CISOFileBlockDevice(FileLoader *fileLoader)
 	CISO_H hdr;
 	size_t readSize = fileLoader->ReadAt(0, sizeof(CISO_H), 1, &hdr);
 	if (readSize != 1 || memcmp(hdr.magic, "CISO", 4) != 0) {
-		WARN_LOG(LOADER, "Invalid CSO!");
+		errorString_ = "Invalid CSO!";
+		return;
 	}
 	if (hdr.ver > 1) {
-		WARN_LOG(LOADER, "CSO version too high!");
+		errorString_ = "CSO version too high!";
+		return;
 	}
 
 	frameSize = hdr.block_size;
-	if ((frameSize & (frameSize - 1)) != 0)
-		ERROR_LOG(LOADER, "CSO block size %i unsupported, must be a power of two", frameSize);
-	else if (frameSize < 0x800)
-		ERROR_LOG(LOADER, "CSO block size %i unsupported, must be at least one sector", frameSize);
+	if ((frameSize & (frameSize - 1)) != 0) {
+		errorString_ = StringFromFormat("CSO block size %i unsupported, must be a power of two", frameSize);
+		return;
+	} else if (frameSize < 0x800) {
+		errorString_ = StringFromFormat("CSO block size %i unsupported, must be at least one sector", frameSize);
+		return;
+	}
 
 	// Determine the translation from block to frame.
 	blockShift = 0;
@@ -166,7 +182,7 @@ CISOFileBlockDevice::CISOFileBlockDevice(FileLoader *fileLoader)
 	const u64 totalSize = hdr.total_bytes;
 	numFrames = (u32)((totalSize + frameSize - 1) / frameSize);
 	numBlocks = (u32)(totalSize / GetBlockSize());
-	VERBOSE_LOG(LOADER, "CSO numBlocks=%i numFrames=%i align=%i", numBlocks, numFrames, indexShift);
+	VERBOSE_LOG(Log::Loader, "CSO numBlocks=%i numFrames=%i align=%i", numBlocks, numFrames, indexShift);
 
 	// We might read a bit of alignment too, so be prepared.
 	if (frameSize + (1 << indexShift) < CSO_READ_BUFFER_SIZE)
@@ -207,10 +223,12 @@ CISOFileBlockDevice::CISOFileBlockDevice(FileLoader *fileLoader)
 	u64 lastIndexPos = index[indexSize - 1] & 0x7FFFFFFF;
 	u64 expectedFileSize = lastIndexPos << indexShift;
 	if (expectedFileSize > fileSize) {
-		ERROR_LOG(LOADER, "Expected CSO to at least be %lld bytes, but file is %lld bytes. File: '%s'",
-			expectedFileSize, fileSize, fileLoader->GetPath().c_str());
-		NotifyReadError();
+		errorString_ = StringFromFormat("Expected CSO to at least be %lld bytes, but file is %lld bytes", expectedFileSize, fileSize);
+		return;
 	}
+
+	// all ok.
+	_dbg_assert_(errorString_.empty());
 }
 
 CISOFileBlockDevice::~CISOFileBlockDevice()
@@ -258,7 +276,7 @@ bool CISOFileBlockDevice::ReadBlock(int blockNumber, u8 *outPtr, bool uncached)
 		z.zfree = Z_NULL;
 		z.opaque = Z_NULL;
 		if (inflateInit2(&z, -15) != Z_OK) {
-			ERROR_LOG(LOADER, "GetBlockSize() ERROR: %s\n", (z.msg) ? z.msg : "?");
+			ERROR_LOG(Log::Loader, "GetBlockSize() ERROR: %s\n", (z.msg) ? z.msg : "?");
 			NotifyReadError();
 			return false;
 		}
@@ -269,14 +287,14 @@ bool CISOFileBlockDevice::ReadBlock(int blockNumber, u8 *outPtr, bool uncached)
 
 		int status = inflate(&z, Z_FINISH);
 		if (status != Z_STREAM_END) {
-			ERROR_LOG(LOADER, "block %d: inflate : %s[%d]\n", blockNumber, (z.msg) ? z.msg : "error", status);
+			ERROR_LOG(Log::Loader, "block %d: inflate : %s[%d]\n", blockNumber, (z.msg) ? z.msg : "error", status);
 			NotifyReadError();
 			inflateEnd(&z);
 			memset(outPtr, 0, GetBlockSize());
 			return false;
 		}
 		if (z.total_out != frameSize) {
-			ERROR_LOG(LOADER, "block %d: block size error %d != %d\n", blockNumber, (u32)z.total_out, frameSize);
+			ERROR_LOG(Log::Loader, "block %d: block size error %d != %d\n", blockNumber, (u32)z.total_out, frameSize);
 			NotifyReadError();
 			inflateEnd(&z);
 			memset(outPtr, 0, GetBlockSize());
@@ -314,7 +332,7 @@ bool CISOFileBlockDevice::ReadBlocks(u32 minBlock, int count, u8 *outPtr) {
 
 	z_stream z{};
 	if (inflateInit2(&z, -15) != Z_OK) {
-		ERROR_LOG(LOADER, "Unable to initialize inflate: %s\n", (z.msg) ? z.msg : "?");
+		ERROR_LOG(Log::Loader, "Unable to initialize inflate: %s\n", (z.msg) ? z.msg : "?");
 		return false;
 	}
 
@@ -358,11 +376,11 @@ bool CISOFileBlockDevice::ReadBlocks(u32 minBlock, int count, u8 *outPtr) {
 
 			int status = inflate(&z, Z_FINISH);
 			if (status != Z_STREAM_END) {
-				ERROR_LOG(LOADER, "Inflate frame %d: failed - %s[%d]\n", frame, (z.msg) ? z.msg : "error", status);
+				ERROR_LOG(Log::Loader, "Inflate frame %d: failed - %s[%d]\n", frame, (z.msg) ? z.msg : "error", status);
 				NotifyReadError();
 				memset(outPtr, 0, frameBlocks * GetBlockSize());
 			} else if (z.total_out != frameSize) {
-				ERROR_LOG(LOADER, "Inflate frame %d: block size error %d != %d\n", frame, (u32)z.total_out, frameSize);
+				ERROR_LOG(Log::Loader, "Inflate frame %d: block size error %d != %d\n", frame, (u32)z.total_out, frameSize);
 				NotifyReadError();
 				memset(outPtr, 0, frameBlocks * GetBlockSize());
 			} else if (frameBlocks != blocksPerFrame) {
@@ -385,55 +403,83 @@ bool CISOFileBlockDevice::ReadBlocks(u32 minBlock, int count, u8 *outPtr) {
 NPDRMDemoBlockDevice::NPDRMDemoBlockDevice(FileLoader *fileLoader)
 	: BlockDevice(fileLoader)
 {
-	std::lock_guard<std::mutex> guard(mutex_);
 	MAC_KEY mkey;
 	CIPHER_KEY ckey;
 	u8 np_header[256];
-	u32 tableOffset, tableSize;
-	u32 lbaStart, lbaEnd;
+	u32 tableOffset_, tableSize_;
 
 	fileLoader_->ReadAt(0x24, 1, 4, &psarOffset);
 	size_t readSize = fileLoader_->ReadAt(psarOffset, 1, 256, &np_header);
 	if (readSize != 256){
-		ERROR_LOG(LOADER, "Invalid NPUMDIMG header!");
+		errorString_ = "Invalid NPUMDIMG header!";
+		return;
 	}
 
-	kirk_init();
+	u32 psar_id;
+	fileLoader->ReadAt(psarOffset, 4, 1, &psar_id);
+
+	INFO_LOG(Log::Loader, "NPDRM: PSAR ID: %08x", psar_id);
+	// PS1 PSAR begins with "PSISOIMG0000"
+	if (psar_id == 'SISP') {
+		lbaSize_ = 0;  // Mark invalid
+		ERROR_LOG(Log::Loader, "PSX not supported! Should have been caught earlier.");
+		errorString_ = "PSX ISOs not supported!";
+		return;
+	}
+
+	std::lock_guard<std::mutex> guard(mutex_);
+
+	// Local kirk instance to not clash with other block devices and other decryption things.
+	kirk_init(&kirk_);
 
 	// getkey
 	sceDrmBBMacInit(&mkey, 3);
-	sceDrmBBMacUpdate(&mkey, np_header, 0xc0);
-	bbmac_getkey(&mkey, np_header+0xc0, vkey);
+	sceDrmBBMacUpdate(&kirk_, &mkey, np_header, 0xc0);
+	bbmac_getkey(&kirk_, &mkey, np_header+0xc0, vkey);
 
 	// decrypt NP header
 	memcpy(hkey, np_header+0xa0, 0x10);
-	sceDrmBBCipherInit(&ckey, 1, 2, hkey, vkey, 0);
-	sceDrmBBCipherUpdate(&ckey, np_header+0x40, 0x60);
+	sceDrmBBCipherInit(&kirk_, &ckey, 1, 2, hkey, vkey, 0);
+	sceDrmBBCipherUpdate(&kirk_, &ckey, np_header+0x40, 0x60);
 	sceDrmBBCipherFinal(&ckey);
 
-	lbaStart = *(u32*)(np_header+0x54); // LBA start
-	lbaEnd   = *(u32*)(np_header+0x64); // LBA end
-	lbaSize  = (lbaEnd-lbaStart+1);     // LBA size of ISO
-	blockLBAs = *(u32*)(np_header+0x0c); // block size in LBA
-	blockSize = blockLBAs*2048;
-	numBlocks = (lbaSize+blockLBAs-1)/blockLBAs; // total blocks;
+	u32 lbaStart = *(u32*)(np_header+0x54); // LBA start
+	u32 lbaEnd   = *(u32*)(np_header+0x64); // LBA end
+	lbaSize_     = (lbaEnd-lbaStart+1);     // LBA size of ISO
+	blockLBAs_   = *(u32*)(np_header+0x0c); // block size in LBA
 
-	blockBuf = new u8[blockSize];
-	tempBuf  = new u8[blockSize];
+	char psarStr[5] = {};
+	memcpy(psarStr, &psar_id, 4);
 
-	tableOffset = *(u32*)(np_header+0x6c); // table offset
+	// Protect against a badly decrypted header, and send information through the assert about what's being played (implicitly).
+	_dbg_assert_msg_(blockLBAs_ <= 4096, "Bad blockLBAs in header: %08x (%s) psar: %s", blockLBAs_, fileLoader->GetPath().ToVisualString().c_str(), psarStr);
 
-	tableSize = numBlocks*32;
-	table = new table_info[numBlocks];
-
-	readSize = fileLoader_->ReadAt(psarOffset + tableOffset, 1, tableSize, table);
-	if(readSize!=tableSize){
-		ERROR_LOG(LOADER, "Invalid NPUMDIMG table!");
+	// When we remove the above assert, let's just try to survive.
+	if (blockLBAs_ > 4096) {
+		errorString_ = StringFromFormat("Bad blockLBAs in header: %08x (%s) psar: %s", blockLBAs_, fileLoader->GetPath().ToVisualString().c_str(), psarStr);
+		return;
 	}
 
-	u32 *p = (u32*)table;
+	blockSize_ = blockLBAs_ * 2048;
+	numBlocks_ = (lbaSize_ + blockLBAs_-1) / blockLBAs_; // total blocks;
+
+	blockBuf_ = new u8[blockSize_];
+	tempBuf_  = new u8[blockSize_];
+
+	tableOffset_ = *(u32*)(np_header+0x6c); // table offset
+
+	tableSize_ = numBlocks_ * 32;
+	table_ = new table_info[numBlocks_];
+
+	readSize = fileLoader_->ReadAt(psarOffset + tableOffset_, 1, tableSize_, table_);
+	if (readSize != tableSize_){
+		errorString_ = "Invalid NPUMDIMG table!";
+		return;
+	}
+
+	u32 *p = (u32*)table_;
 	u32 i, k0, k1, k2, k3;
-	for(i=0; i<numBlocks; i++){
+	for (i=0; i<numBlocks_; i++){
 		k0 = p[0]^p[1];
 		k1 = p[1]^p[2];
 		k2 = p[0]^p[3];
@@ -445,86 +491,84 @@ NPDRMDemoBlockDevice::NPDRMDemoBlockDevice(FileLoader *fileLoader)
 		p += 8;
 	}
 
-	currentBlock = -1;
+	currentBlock_ = -1;
+	_dbg_assert_(errorString_.empty());
 }
 
-NPDRMDemoBlockDevice::~NPDRMDemoBlockDevice()
-{
+NPDRMDemoBlockDevice::~NPDRMDemoBlockDevice() {
 	std::lock_guard<std::mutex> guard(mutex_);
-	delete [] table;
-	delete [] tempBuf;
-	delete [] blockBuf;
+	delete [] table_;
+	delete [] tempBuf_;
+	delete [] blockBuf_;
 }
 
 int lzrc_decompress(void *out, int out_len, void *in, int in_len);
 
-bool NPDRMDemoBlockDevice::ReadBlock(int blockNumber, u8 *outPtr, bool uncached)
-{
+bool NPDRMDemoBlockDevice::ReadBlock(int blockNumber, u8 *outPtr, bool uncached) {
 	FileLoader::Flags flags = uncached ? FileLoader::Flags::HINT_UNCACHED : FileLoader::Flags::NONE;
 	std::lock_guard<std::mutex> guard(mutex_);
-	CIPHER_KEY ckey;
-	int block, lba, lzsize;
-	size_t readSize;
-	u8 *readBuf;
 
-	lba = blockNumber-currentBlock;
-	if(lba>=0 && lba<blockLBAs){
-		memcpy(outPtr, blockBuf+lba*2048, 2048);
+	if (blockSize_ == 0) {
+		// Wasn't opened successfully.
+		return false;
+	}
+
+	int lba = blockNumber - currentBlock_;
+	if (lba >= 0 && lba < blockLBAs_){
+		memcpy(outPtr, blockBuf_ + lba*2048, 2048);
 		return true;
 	}
 
-	block = blockNumber/blockLBAs;
-	lba = blockNumber%blockLBAs;
-	currentBlock = block*blockLBAs;
+	int block = blockNumber / blockLBAs_;
+	lba = blockNumber % blockLBAs_;
+	currentBlock_ = block * blockLBAs_;
 
-	if(table[block].unk_1c!=0){
-		if((u32)block==(numBlocks-1))
+	if (table_[block].unk_1c != 0) {
+		if((u32)block == (numBlocks_ - 1))
 			return true; // demos make by fake_np
 		else
 			return false;
 	}
 
-	if(table[block].size<blockSize)
-		readBuf = tempBuf;
+	u8 *readBuf;
+	if (table_[block].size < blockSize_)
+		readBuf = tempBuf_;
 	else
-		readBuf = blockBuf;
+		readBuf = blockBuf_;
 
-	readSize = fileLoader_->ReadAt(psarOffset+table[block].offset, 1, table[block].size, readBuf, flags);
-	if(readSize != (size_t)table[block].size){
-		if((u32)block==(numBlocks-1))
+	size_t readSize = fileLoader_->ReadAt(psarOffset+table_[block].offset, 1, table_[block].size, readBuf, flags);
+	if (readSize != (size_t)table_[block].size){
+		if((u32)block==(numBlocks_-1))
 			return true;
 		else
 			return false;
 	}
 
-	if((table[block].flag&1)==0){
+	if ((table_[block].flag & 1) == 0) {
 		// skip mac check
 	}
 
-	if((table[block].flag&4)==0){
-		sceDrmBBCipherInit(&ckey, 1, 2, hkey, vkey, table[block].offset>>4);
-		sceDrmBBCipherUpdate(&ckey, readBuf, table[block].size);
+	if ((table_[block].flag & 4) == 0) {
+		CIPHER_KEY ckey;
+		sceDrmBBCipherInit(&kirk_, &ckey, 1, 2, hkey, vkey, table_[block].offset>>4);
+		sceDrmBBCipherUpdate(&kirk_, &ckey, readBuf, table_[block].size);
 		sceDrmBBCipherFinal(&ckey);
 	}
 
-	if(table[block].size<blockSize){
-		lzsize = lzrc_decompress(blockBuf, 0x00100000, readBuf, table[block].size);
-		if(lzsize!=blockSize){
-			ERROR_LOG(LOADER, "LZRC decompress error! lzsize=%d\n", lzsize);
+	if (table_[block].size < blockSize_) {
+		int lzsize = lzrc_decompress(blockBuf_, 0x00100000, readBuf, table_[block].size);
+		if(lzsize != blockSize_){
+			ERROR_LOG(Log::Loader, "LZRC decompress error! lzsize=%d\n", lzsize);
 			NotifyReadError();
 			return false;
 		}
 	}
 
-	memcpy(outPtr, blockBuf+lba*2048, 2048);
-
+	memcpy(outPtr, blockBuf_+lba*2048, 2048);
 	return true;
 }
 
-/*
- * CHD file
- */
-static const UINT8 nullsha1[CHD_SHA1_BYTES] = { 0 };
+// static const UINT8 nullsha1[CHD_SHA1_BYTES] = { 0 };
 
 struct CHDImpl {
 	chd_file *chd = nullptr;
@@ -537,8 +581,7 @@ struct ExtendedCoreFile {
 };
 
 CHDFileBlockDevice::CHDFileBlockDevice(FileLoader *fileLoader)
-	: BlockDevice(fileLoader), impl_(new CHDImpl())
-{
+	: BlockDevice(fileLoader), impl_(new CHDImpl()) {
 	Path paths[8];
 	paths[0] = fileLoader->GetPath();
 	int depth = 0;
@@ -593,7 +636,7 @@ CHDFileBlockDevice::CHDFileBlockDevice(FileLoader *fileLoader)
 
 	chd_error err = chd_read_header(paths[0].c_str(), &childHeader);
 	if (err != CHDERR_NONE) {
-		ERROR_LOG(LOADER, "Error loading CHD header for '%s': %s", paths[0].c_str(), chd_error_string(err));
+		ERROR_LOG(Log::Loader, "Error loading CHD header for '%s': %s", paths[0].c_str(), chd_error_string(err));
 		NotifyReadError();
 		return;
 	}
@@ -616,7 +659,7 @@ CHDFileBlockDevice::CHDFileBlockDevice(FileLoader *fileLoader)
 
 				if (chd_read_header(filepath.c_str(), &parentHeader) == CHDERR_NONE &&
 					memcmp(parentHeader.sha1, childHeader.parentsha1, sizeof(parentHeader.sha1)) == 0) {
-					// ERROR_LOG(LOADER, "Checking '%s'", filepath.c_str());
+					// ERROR_LOG(Log::Loader, "Checking '%s'", filepath.c_str());
 					paths[++depth] = filepath;
 					break;
 				}
@@ -624,7 +667,7 @@ CHDFileBlockDevice::CHDFileBlockDevice(FileLoader *fileLoader)
 
 			// Check if parentHeader was opened
 			if (parentHeader.length == 0) {
-				ERROR_LOG(LOADER, "Error loading CHD '%s': parents not found", fileLoader->GetPath().c_str());
+				ERROR_LOG(Log::Loader, "Error loading CHD '%s': parents not found", fileLoader->GetPath().c_str());
 				NotifyReadError();
 				return;
 			}
@@ -636,8 +679,7 @@ CHDFileBlockDevice::CHDFileBlockDevice(FileLoader *fileLoader)
 	chd_file *file = nullptr;
 	chd_error err = chd_open_core_file(&core_file_->core, CHD_OPEN_READ, NULL, &file);
 	if (err != CHDERR_NONE) {
-		ERROR_LOG(LOADER, "Error loading CHD '%s': %s", paths[depth].c_str(), chd_error_string(err));
-		NotifyReadError();
+		errorString_ = StringFromFormat("CHD error: %s", paths[depth].c_str(), chd_error_string(err));
 		return;
 	}
 
@@ -648,20 +690,20 @@ CHDFileBlockDevice::CHDFileBlockDevice(FileLoader *fileLoader)
 	currentHunk = -1;
 	blocksPerHunk = impl_->header->hunkbytes / impl_->header->unitbytes;
 	numBlocks = impl_->header->unitcount;
+
+	_dbg_assert_(errorString_.empty());
 }
 
-CHDFileBlockDevice::~CHDFileBlockDevice()
-{
+CHDFileBlockDevice::~CHDFileBlockDevice() {
 	if (impl_->chd) {
 		chd_close(impl_->chd);
 		delete[] readBuffer;
 	}
 }
 
-bool CHDFileBlockDevice::ReadBlock(int blockNumber, u8 *outPtr, bool uncached)
-{
+bool CHDFileBlockDevice::ReadBlock(int blockNumber, u8 *outPtr, bool uncached) {
 	if (!impl_->chd) {
-		ERROR_LOG(LOADER, "ReadBlock: CHD not open. %s", fileLoader_->GetPath().c_str());
+		ERROR_LOG(Log::Loader, "ReadBlock: CHD not open. %s", fileLoader_->GetPath().c_str());
 		return false;
 	}
 	if ((u32)blockNumber >= numBlocks) {
@@ -674,7 +716,7 @@ bool CHDFileBlockDevice::ReadBlock(int blockNumber, u8 *outPtr, bool uncached)
 	if (currentHunk != hunk) {
 		chd_error err = chd_read(impl_->chd, hunk, readBuffer);
 		if (err != CHDERR_NONE) {
-			ERROR_LOG(LOADER, "CHD read failed: %d %d %s", blockNumber, hunk, chd_error_string(err));
+			ERROR_LOG(Log::Loader, "CHD read failed: %d %d %s", blockNumber, hunk, chd_error_string(err));
 			NotifyReadError();
 		}
 		currentHunk = hunk;
