@@ -16,15 +16,14 @@
 // https://github.com/hrydgard/ppsspp and http://www.ppsspp.org/.
 
 #include "ppsspp_config.h"
+
 #include <algorithm>
 #include <map>
 #include <unordered_map>
 
 #include "Common/CommonTypes.h"
-#include "Common/Data/Convert/SmallDataConvert.h"
 #include "Common/Log.h"
 #include "Common/Swap.h"
-#include "Core/Config.h"
 #include "Core/System.h"
 #include "Core/Debugger/Breakpoints.h"
 #include "Core/Debugger/MemBlockInfo.h"
@@ -39,12 +38,8 @@
 
 #include "GPU/Math3D.h"
 #include "GPU/GPU.h"
-#include "GPU/GPUInterface.h"
-#include "GPU/GPUState.h"
-
-#if PPSSPP_ARCH(X86) || PPSSPP_ARCH(AMD64)
-#include <emmintrin.h>
-#endif
+#include "GPU/GPUCommon.h"
+#include "Common/Math/SIMDHeaders.h"
 
 enum class GPUReplacementSkip {
 	MEMSET = 1,
@@ -1285,6 +1280,110 @@ static int Hook_omertachinmokunookitethelegacy_download_frame() {
 	return 0;
 }
 
+// Function at 0886665C in US version (Persona 1)
+// Function at 08807DC4 in EU version (Persona 2)
+static int Hook_persona_download_frame() {
+	// Depending on a global (curframe kind of thing), this either reads from
+	// 0x04088000 or 0x04000000 (the two addresses are hardcoded).
+	// We'd have to do some gnarly stuff to get this address, so let's just download both.
+	for (int i = 0; i < 2; i++) {
+		const u32 fb_address = i == 0 ? 0x04000000 : 0x04088000;
+		gpu->PerformReadbackToMemory(fb_address, 0x00088000);
+		NotifyMemInfo(MemBlockFlags::WRITE, fb_address, 0x00088000, "persona1_download_frame");
+	}
+	return 0;
+}
+
+static int Hook_steinsgate_download_frame() {
+	u32 fb_offset_addr;
+	if (!GetMIPSStaticAddress(fb_offset_addr, 0x1C, 0x20)) {
+		return 0;
+	}
+	const u32 fb_address = 0x04000000 + Memory::Read_U32(fb_offset_addr);
+	if (Memory::IsVRAMAddress(fb_address)) {
+		gpu->PerformReadbackToMemory(fb_address, 0x00088000);
+		NotifyMemInfo(MemBlockFlags::WRITE, fb_address, 0x00088000, "steinsgate_download_frame");
+	}
+	return 0;
+}
+
+static int Hook_infinity_download_frame() {
+	// There are a few games that share this same function.
+	// The hash matches, but due to relocations, the addresses it references differ.
+	// Because of this, the address, even though hardcoded, has to be fetched from the function.
+	u32 magic_value_addr;
+	if (!GetMIPSStaticAddress(magic_value_addr, 0x08, 0x1C)) {
+		return 0;
+	}
+
+	// Not sure why it was done like this, but that's what the actual function does.
+	const u32 fb_address = (Memory::Read_U32(magic_value_addr) & 1) ? 0x04000000 : 0x04088000;
+
+	gpu->PerformReadbackToMemory(fb_address, 0x00088000);
+	NotifyMemInfo(MemBlockFlags::WRITE, fb_address, 0x00088000, "infinity_download_frame");
+	return 0;
+}
+
+static int Hook_takuyo_download_frame() {
+	gpu->PerformReadbackToMemory(0x04088000, 0x00088000); // The offset is hardcoded.
+	NotifyMemInfo(MemBlockFlags::WRITE, 0x04088000, 0x00088000, "takuyo_download_frame");
+	return 0;
+}
+
+// Offsets in comments are valid for the US version of "KINGDOM HEARTS Birth by Sleep".
+// Function at 0x0881EF68
+static int Hook_kingdomhearts_download_frame() {
+	const u32 fb_base = 0x04000000; // Set in 0x0880C458, doesn't seem like it's ever overwriten.
+
+	const u32 get_fb_offset = MIPSCodeUtils::GetJumpTarget(currentMIPS->pc + 0x5C); // Jump to function at 0x08821EB0. Said function returns the framebuffer offset.
+	if (get_fb_offset == INVALIDTARGET) {
+		return 0;
+	}
+	u32 fb_offset_index_addr;
+	if (!GetMIPSStaticAddress(fb_offset_index_addr, get_fb_offset - currentMIPS->pc, get_fb_offset + 0x04 - currentMIPS->pc)) {
+		return 0;
+	}
+	if (!Memory::IsValidRange(fb_offset_index_addr, 4)) {
+		return 0;
+	}
+
+	const u32 fb_offset_index = Memory::Read_U32(fb_offset_index_addr); // 0x08821E90-0x08821E98
+	if (fb_offset_index > 2) {
+		return 0;
+	}
+
+	const MIPSOpcode fb_offset_table_lui = Memory::Read_Instruction(get_fb_offset + 0x08, true); // 0x08821EB8
+	if (fb_offset_table_lui != MIPS_MAKE_LUI(MIPS_REG_A1, fb_offset_table_lui & 0xFFFF)) {
+		return 0;
+	}
+	const MIPSOpcode fb_offset_table_addiu = Memory::Read_Instruction(get_fb_offset + 0x10, true); // 0x08821EC0
+	if (fb_offset_table_addiu != MIPS_MAKE_ADDIU(MIPS_REG_A1, MIPS_REG_A1, fb_offset_table_addiu & 0xFFFF)) {
+		return 0;
+	}
+	const u32 fb_offset_table = ((fb_offset_table_lui & 0xFFFF) << 16) + (s16)(fb_offset_table_addiu & 0xFFFF);
+	if (!Memory::IsValidRange(fb_offset_table, 12)) {
+		return 0;
+	}
+	const u32 fb_offset = Memory::Read_U32(fb_offset_table + fb_offset_index*4); // 0x08821E98-0x08821EB0
+
+	u32 magic_ptr_addr;
+	if (!GetMIPSStaticAddress(magic_ptr_addr, 0x08, 0x10)) {
+		return 0;
+	}
+	const u32 magic_ptr = Memory::Read_U32(magic_ptr_addr); // 0x0881EF70, 0x0881EF78
+
+	// Function of the variable guessed.
+	const u8 bytes_per_pixel = Memory::Read_U8(magic_ptr+0x50); // 0x0881EFE0
+
+	const u32 fb_address = fb_base + fb_offset;
+	const u32 fb_size = (bytes_per_pixel == 2) ? 0x044000 : 0x088000; // Branch at 0x0881EFE8, s3 set at 0x0881EFB8
+	if (Memory::IsVRAMAddress(fb_address)) {
+		gpu->PerformReadbackToMemory(fb_address, fb_size);
+		NotifyMemInfo(MemBlockFlags::WRITE, fb_address, fb_size, "kingdomhearts_download_frame");
+	}
+	return 0;
+}
+
 static int Hook_katamari_render_check() {
 	const u32 fb_address = Memory::Read_U32(currentMIPS->r[MIPS_REG_A0] + 0x3C);
 	const u32 fbInfoPtr = Memory::Read_U32(currentMIPS->r[MIPS_REG_A0] + 0x40);
@@ -1377,7 +1476,7 @@ static int Hook_starocean_clear_framebuf_after() {
 		int y = (s16)Memory::Read_U16(y_address);
 		int h = (s16)Memory::Read_U16(h_address);
 
-		DEBUG_LOG(HLE, "starocean_clear_framebuf() - %08x y=%d-%d", framebuf, y, h);
+		DEBUG_LOG(Log::HLE, "starocean_clear_framebuf() - %08x y=%d-%d", framebuf, y, h);
 		// TODO: This is always clearing to 0, actually, which could be faster than an upload.
 		gpu->PerformWriteColorFromMemory(framebuf + 512 * y * 4, 512 * h * 4);
 	}
@@ -1435,8 +1534,8 @@ static int Hook_soltrigger_render_ucschar() {
 }
 
 static int Hook_gow_fps_hack() {
-	if (PSP_CoreParameter().compat.flags().GoWFramerateHack60 || PSP_CoreParameter().compat.flags().GoWFramerateHack30) {
-		if (PSP_CoreParameter().compat.flags().GoWFramerateHack30) {
+	if (PSP_CoreParameter().compat.flags().GoWFramerateHack60 || PSP_CoreParameter().compat.flags().FramerateHack30) {
+		if (PSP_CoreParameter().compat.flags().FramerateHack30) {
 			__DisplayWaitForVblanks("vblank start waited", 2);
 		} else {
 			__DisplayWaitForVblanks("vblank start waited", 1);
@@ -1445,6 +1544,20 @@ static int Hook_gow_fps_hack() {
 	return 0;
 }
 
+static int Hook_blitz_fps_hack() {
+	if (PSP_CoreParameter().compat.flags().FramerateHack30) {
+		__DisplayWaitForVblanks("vblank start waited", 1);
+	}
+	return 0;
+}
+
+static int Hook_brian_lara_fps_hack() {
+	if (PSP_CoreParameter().compat.flags().FramerateHack30) {
+		__DisplayWaitForVblanks("vblank start waited", 1);
+	}
+	return 0;
+}
+
 static int Hook_gow_vortex_hack() {
 	if (PSP_CoreParameter().compat.flags().GoWFramerateHack60) {
 		// from my tests both ==0x3F800000 and !=0x3F800000 takes around 1:40-1:50, that seems to match correct behaviour
@@ -1584,6 +1697,16 @@ static const ReplacementTableEntry entries[] = {
 	{ "gow_fps_hack", &Hook_gow_fps_hack, 0, REPFLAG_HOOKEXIT , 0 },
 	{ "gow_vortex_hack", &Hook_gow_vortex_hack, 0, REPFLAG_HOOKENTER, 0x60 },
 	{ "ZZT3_select_hack", &Hook_ZZT3_select_hack, 0, REPFLAG_HOOKENTER, 0xC4 },
+	{ "blitz_fps_hack", &Hook_blitz_fps_hack, 0, REPFLAG_HOOKEXIT , 0 },
+	{ "brian_lara_fps_hack", &Hook_brian_lara_fps_hack, 0, REPFLAG_HOOKEXIT , 0 },
+	{ "persona1_download_frame", &Hook_persona_download_frame, 0, REPFLAG_HOOKENTER, 0 },
+	{ "persona2_download_frame", &Hook_persona_download_frame, 0, REPFLAG_HOOKENTER, 0 },
+	{ "steinsgate_download_frame", &Hook_steinsgate_download_frame, 0, REPFLAG_HOOKENTER, 0 },
+	{ "infinity_download_frame", &Hook_infinity_download_frame, 0, REPFLAG_HOOKENTER, 0 },
+	{ "takuyo_1_download_frame", &Hook_takuyo_download_frame, 0, REPFLAG_HOOKENTER, 0},
+	{ "takuyo_2_download_frame", &Hook_takuyo_download_frame, 0, REPFLAG_HOOKENTER, 0},
+	{ "takuyo_3_download_frame", &Hook_takuyo_download_frame, 0, REPFLAG_HOOKENTER, 0},
+	{ "kingdomhearts_download_frame", &Hook_kingdomhearts_download_frame, 0, REPFLAG_HOOKENTER, 0},
 	{}
 };
 
@@ -1639,13 +1762,13 @@ static bool WriteReplaceInstruction(u32 address, int index) {
 		if (prevIndex == index) {
 			return false;
 		}
-		WARN_LOG(HLE, "Replacement func changed at %08x (%d -> %d)", address, prevIndex, index);
+		WARN_LOG(Log::HLE, "Replacement func changed at %08x (%d -> %d)", address, prevIndex, index);
 		// Make sure we don't save the old replacement.
 		prevInstr = replacedInstructions[address];
 	}
 
 	if (MIPS_IS_RUNBLOCK(Memory::Read_U32(address))) {
-		WARN_LOG(HLE, "Replacing jitted func address %08x", address);
+		WARN_LOG(Log::HLE, "Replacing jitted func address %08x", address);
 	}
 	replacedInstructions[address] = prevInstr;
 	Memory::Write_U32(MIPS_EMUHACK_CALL_REPLACEMENT | index, address);
@@ -1656,7 +1779,7 @@ void WriteReplaceInstructions(u32 address, u64 hash, int size) {
 	std::vector<int> indexes = GetReplacementFuncIndexes(hash, size);
 	for (int index : indexes) {
 		bool didReplace = false;
-		auto entry = GetReplacementFunc(index);
+		const ReplacementTableEntry *entry = GetReplacementFunc(index);
 		if (entry->flags & REPFLAG_HOOKEXIT) {
 			// When hooking func exit, we search for jr ra, and replace those.
 			for (u32 offset = 0; offset < (u32)size; offset += 4) {
@@ -1678,7 +1801,7 @@ void WriteReplaceInstructions(u32 address, u64 hash, int size) {
 		}
 
 		if (didReplace) {
-			INFO_LOG(HLE, "Replaced %s at %08x with hash %016llx", entries[index].name, address, hash);
+			INFO_LOG(Log::HLE, "Replaced %s at %08x with hash %016llx", entries[index].name, address, hash);
 		}
 	}
 }
@@ -1687,9 +1810,9 @@ void RestoreReplacedInstruction(u32 address) {
 	const u32 curInstr = Memory::Read_U32(address);
 	if (MIPS_IS_REPLACEMENT(curInstr)) {
 		Memory::Write_U32(replacedInstructions[address], address);
-		NOTICE_LOG(HLE, "Restored replaced func at %08x", address);
+		NOTICE_LOG(Log::HLE, "Restored replaced func at %08x", address);
 	} else {
-		NOTICE_LOG(HLE, "Replaced func changed at %08x", address);
+		NOTICE_LOG(Log::HLE, "Replaced func changed at %08x", address);
 	}
 	replacedInstructions.erase(address);
 }
@@ -1711,29 +1834,27 @@ void RestoreReplacedInstructions(u32 startAddr, u32 endAddr) {
 			++restored;
 		}
 	}
-	INFO_LOG(HLE, "Restored %d replaced funcs between %08x-%08x", restored, startAddr, endAddr);
+	INFO_LOG(Log::HLE, "Restored %d replaced funcs between %08x-%08x", restored, startAddr, endAddr);
 	replacedInstructions.erase(start, end);
 }
 
 std::map<u32, u32> SaveAndClearReplacements() {
 	std::map<u32, u32> saved;
-	for (auto it = replacedInstructions.begin(), end = replacedInstructions.end(); it != end; ++it) {
-		const u32 addr = it->first;
+	for (const auto &[addr, instr] : replacedInstructions) {
 		// This will not retain jit blocks.
 		const u32 curInstr = Memory::Read_Opcode_JIT(addr).encoding;
 		if (MIPS_IS_REPLACEMENT(curInstr)) {
 			saved[addr] = curInstr;
-			Memory::Write_U32(it->second, addr);
+			Memory::Write_U32(instr, addr);
 		}
 	}
 	return saved;
 }
 
 void RestoreSavedReplacements(const std::map<u32, u32> &saved) {
-	for (auto it = saved.begin(), end = saved.end(); it != end; ++it) {
-		const u32 addr = it->first;
+	for (const auto &[addr, instr] : saved) {
 		// Just put the replacements back.
-		Memory::Write_U32(it->second, addr);
+		Memory::Write_U32(instr, addr);
 	}
 }
 
@@ -1759,12 +1880,12 @@ bool CanReplaceJalTo(u32 dest, const ReplacementTableEntry **entry, u32 *funcSiz
 	// Make sure we don't replace if there are any breakpoints inside.
 	*funcSize = g_symbolMap->GetFunctionSize(dest);
 	if (*funcSize == SymbolMap::INVALID_ADDRESS) {
-		if (CBreakPoints::IsAddressBreakPoint(dest)) {
+		if (g_breakpoints.IsAddressBreakPoint(dest)) {
 			return false;
 		}
 		*funcSize = (u32)sizeof(u32);
 	} else {
-		if (CBreakPoints::RangeContainsBreakPoint(dest, *funcSize)) {
+		if (g_breakpoints.RangeContainsBreakPoint(dest, *funcSize)) {
 			return false;
 		}
 	}
@@ -1772,7 +1893,7 @@ bool CanReplaceJalTo(u32 dest, const ReplacementTableEntry **entry, u32 *funcSiz
 	int index = op.encoding & MIPS_EMUHACK_VALUE_MASK;
 	*entry = GetReplacementFunc(index);
 	if (!*entry) {
-		ERROR_LOG(HLE, "ReplaceJalTo: Invalid replacement op %08x at %08x", op.encoding, dest);
+		ERROR_LOG(Log::HLE, "ReplaceJalTo: Invalid replacement op %08x at %08x", op.encoding, dest);
 		return false;
 	}
 
