@@ -76,7 +76,7 @@ void SingleInputMappingFromPspButton(int btn, std::vector<InputMapping> *mapping
 		if (!mapping.empty()) {
 			mappings->push_back(mapping.mappings[0]);
 		} else {
-			WARN_LOG(COMMON, "Encountered empty mapping in multi-mapping for button %d", btn);
+			WARN_LOG(Log::Common, "Encountered empty mapping in multi-mapping for button %d", btn);
 		}
 	}
 }
@@ -225,7 +225,7 @@ static const KeyMap_IntStrPair key_names[] = {
 	{NKCODE_APOSTROPHE, "'"},
 	{NKCODE_MINUS, "-"},
 	{NKCODE_PLUS, "+"},
-	{NKCODE_SYSRQ, "Print"},
+	{NKCODE_PRINTSCREEN, "Print"},
 	{NKCODE_SCROLL_LOCK, "ScrLock"},
 	{NKCODE_BREAK, "Pause"},
 
@@ -238,6 +238,8 @@ static const KeyMap_IntStrPair key_names[] = {
 	{NKCODE_CTRL_RIGHT, "RCtrl"},
 	{NKCODE_ALT_LEFT, "LAlt"},
 	{NKCODE_ALT_RIGHT, "RAlt"},
+	{NKCODE_META_LEFT, "LMeta"},
+	{NKCODE_META_RIGHT, "RMeta"},
 	{NKCODE_SPACE, "Space"},
 	{NKCODE_WINDOW, "Windows"},
 	{NKCODE_DEL, "Backspace"},
@@ -348,20 +350,12 @@ static const KeyMap_IntStrPair key_names[] = {
 	{NKCODE_EXT_MOUSEWHEEL_UP, "MWheelU"},
 	{NKCODE_EXT_MOUSEWHEEL_DOWN, "MWheelD"},
 
-
-	{NKCODE_EXT_MOTION_UP, "MotionUp"},
-	{NKCODE_EXT_MOTION_DOWN, "MotionDown"},
-	{NKCODE_EXT_MOTION_LEFT, "MotionLeft"},
-	{NKCODE_EXT_MOTION_RIGHT, "MotionRight"},
-	{NKCODE_EXT_MOTION_FORWARD, "MotionFwd"},
-	{NKCODE_EXT_ROTATION_UP, "RotationUp"},
-	{NKCODE_EXT_ROTATION_DOWN, "RotationDown"},
-	{NKCODE_EXT_ROTATION_LEFT, "RotationLeft"},
-	{NKCODE_EXT_ROTATION_RIGHT, "RotationRight"},
-
 	{NKCODE_START_QUESTION, "¿"},
 	{NKCODE_LEFTBRACE, "{"},
 	{NKCODE_RIGHTBRACE, "}"},
+
+	{NKCODE_GUIDE, "Guide"},
+	{NKCODE_INFO, "Info"},
 };
 
 static const KeyMap_IntStrPair axis_names[] = {
@@ -426,11 +420,14 @@ const KeyMap_IntStrPair psp_button_names[] = {
 	{VIRTKEY_AXIS_SWAP, "AxisSwap"},
 
 	{VIRTKEY_FASTFORWARD, "Fast-forward"},
+	{VIRTKEY_PAUSE, "Pause"},
+	{VIRTKEY_PAUSE_NO_MENU, "Pause (no menu)"},
+
 	{VIRTKEY_SPEED_TOGGLE, "SpeedToggle"},
 	{VIRTKEY_SPEED_CUSTOM1, "Alt speed 1"},
 	{VIRTKEY_SPEED_CUSTOM2, "Alt speed 2"},
 	{VIRTKEY_SPEED_ANALOG, "Analog speed"},
-	{VIRTKEY_PAUSE, "Pause"},
+	{VIRTKEY_RESET_EMULATION, "Reset"},
 	{VIRTKEY_FRAME_ADVANCE, "Frame Advance"},
 #if !defined(MOBILE_DEVICE)
 	{VIRTKEY_RECORD, "Audio/Video Recording" },
@@ -443,6 +440,7 @@ const KeyMap_IntStrPair psp_button_names[] = {
 #if !defined(MOBILE_DEVICE)
 	{VIRTKEY_TOGGLE_FULLSCREEN, "Toggle Fullscreen"},
 #endif
+	{VIRTKEY_TOGGLE_DEBUGGER, "Toggle Debugger"},
 
 	{VIRTKEY_OPENCHAT, "OpenChat" },
 
@@ -481,6 +479,10 @@ const KeyMap_IntStrPair psp_button_names[] = {
 	{CTRL_VOL_DOWN, "Vol -"},
 	{CTRL_SCREEN, "Screen"},
 	{CTRL_NOTE, "Note"},
+	{CTRL_L2, "Dev-kit L2"},
+	{CTRL_L3, "Dev-kit L3"},
+	{CTRL_R2, "Dev-kit R2"},
+	{CTRL_R3, "Dev-kit R3"},
 };
 
 // key here can be other things than InputKeyCode.
@@ -730,17 +732,17 @@ void RestoreDefault() {
 	std::string name = System_GetProperty(SYSPROP_NAME);
 	if (IsNvidiaShield(name)) {
 		SetDefaultKeyMap(DEFAULT_MAPPING_SHIELD, false);
-	} else if (IsOuya(name)) {
-		SetDefaultKeyMap(DEFAULT_MAPPING_OUYA, false);
 	} else if (IsXperiaPlay(name)) {
 		SetDefaultKeyMap(DEFAULT_MAPPING_XPERIA_PLAY, false);
 	} else if (IsMOQII7S(name)) {
 		SetDefaultKeyMap(DEFAULT_MAPPING_MOQI_I7S, false);
 	} else if (IsRetroid(name)) {
-		SetDefaultKeyMap(DEFAULT_MAPPING_RETRO_STATION_CONTROLLER, false);
+		SetDefaultKeyMap(DEFAULT_MAPPING_RETROID_CONTROLLER, false);
 	} else {
 		SetDefaultKeyMap(DEFAULT_MAPPING_ANDROID_PAD, false);
 	}
+#elif PPSSPP_PLATFORM(IOS)
+	SetDefaultKeyMap(DEFAULT_MAPPING_IOS_PAD, false);
 #else
 	SetDefaultKeyMap(DEFAULT_MAPPING_KEYBOARD, true);
 	SetDefaultKeyMap(DEFAULT_MAPPING_PAD, false);
@@ -759,7 +761,7 @@ void LoadFromIni(IniFile &file) {
 	Section *controls = file.GetOrCreateSection("ControlMapping");
 	for (size_t i = 0; i < ARRAY_SIZE(psp_button_names); i++) {
 		std::string value;
-		controls->Get(psp_button_names[i].name, &value, "");
+		controls->Get(psp_button_names[i].name, &value);
 
 		// Erase default mapping
 		g_controllerMap.erase(psp_button_names[i].key);
@@ -810,10 +812,6 @@ void ClearAllMappings() {
 	g_controllerMapGeneration++;
 }
 
-bool IsOuya(const std::string &name) {
-	return name == "OUYA:OUYA Console";
-}
-
 bool IsNvidiaShield(const std::string &name) {
 	return name == "NVIDIA:SHIELD";
 }
@@ -837,7 +835,7 @@ bool IsMOQII7S(const std::string &name) {
 }
 
 bool HasBuiltinController(const std::string &name) {
-	return IsOuya(name) || IsXperiaPlay(name) || IsNvidiaShield(name) || IsMOQII7S(name) || IsRetroid(name);
+	return IsXperiaPlay(name) || IsNvidiaShield(name) || IsMOQII7S(name) || IsRetroid(name);
 }
 
 void NotifyPadConnected(InputDeviceID deviceId, const std::string &name) {
@@ -850,13 +848,13 @@ void AutoConfForPad(const std::string &name) {
 	std::lock_guard<std::recursive_mutex> guard(g_controllerMapLock);
 	g_controllerMap.clear();
 
-	INFO_LOG(SYSTEM, "Autoconfiguring pad for '%s'", name.c_str());
+	INFO_LOG(Log::System, "Autoconfiguring pad for '%s'", name.c_str());
 
 #if PPSSPP_PLATFORM(ANDROID)
 	if (name.find("Xbox") != std::string::npos) {
 		SetDefaultKeyMap(DEFAULT_MAPPING_ANDROID_XBOX, false);
 	} else if (name == "Retro Station Controller") {
-		SetDefaultKeyMap(DEFAULT_MAPPING_RETRO_STATION_CONTROLLER, false);
+		SetDefaultKeyMap(DEFAULT_MAPPING_RETROID_CONTROLLER, false);
 	} else {
 		SetDefaultKeyMap(DEFAULT_MAPPING_ANDROID_PAD, false);
 	}
