@@ -40,6 +40,7 @@
 #include "Core/FileSystems/MetaFileSystem.h"
 #include "Core/ELF/ParamSFO.h"
 #include "Core/HLE/HLE.h"
+#include "Core/HLE/sceNet.h"
 #include "Core/HLE/ReplaceTables.h"
 #include "Core/HLE/sceDisplay.h"
 #include "Core/HLE/sceKernel.h"
@@ -59,8 +60,10 @@
 // Slot number is visual only, -2 will display special message
 constexpr int LOAD_UNDO_SLOT = -2;
 
-namespace SaveState
-{
+namespace SaveState {
+
+double g_lastSaveTime = -1.0;
+
 	struct SaveStart
 	{
 		void DoState(PointerWrap &p);
@@ -75,19 +78,15 @@ namespace SaveState
 		SAVESTATE_SAVE_SCREENSHOT,
 	};
 
-	struct Operation
-	{
+	struct Operation {
 		// The slot number is for visual purposes only. Set to -1 for operations where we don't display a message for example.
-		Operation(OperationType t, const Path &f, int slot_, Callback cb, void *cbUserData_)
-			: type(t), filename(f), callback(cb), slot(slot_), cbUserData(cbUserData_)
-		{
-		}
+		Operation(OperationType t, const Path &f, int slot_, Callback cb)
+			: type(t), filename(f), callback(cb), slot(slot_) {}
 
 		OperationType type;
 		Path filename;
 		Callback callback;
 		int slot;
-		void *cbUserData;
 	};
 
 	CChunkFileReader::Error SaveToRam(std::vector<u8> &data) {
@@ -211,7 +210,7 @@ namespace SaveState
 			}
 
 			double taken_s = time_now_d() - start_time;
-			DEBUG_LOG(SAVESTATE, "Rewind: Compressed save from %d bytes to %d in %0.2f ms.", (int)state.size(), (int)result.size(), taken_s * 1000.0);
+			DEBUG_LOG(Log::SaveState, "Rewind: Compressed save from %d bytes to %d in %0.2f ms.", (int)state.size(), (int)result.size(), taken_s * 1000.0);
 		}
 
 		void LockedDecompress(std::vector<u8> &result, const std::vector<u8> &compressed, const std::vector<u8> &base)
@@ -275,7 +274,7 @@ namespace SaveState
 			if (g_Config.iRewindSnapshotInterval <= 0) {
 				return;
 			}
-			if (coreState != CORE_RUNNING) {
+			if (coreState != CORE_RUNNING_CPU) {
 				return;
 			}
 
@@ -285,7 +284,7 @@ namespace SaveState
 			if (diff < g_Config.iRewindSnapshotInterval)
 				return;
 
-			DEBUG_LOG(SAVESTATE, "Saving rewind state");
+			DEBUG_LOG(Log::SaveState, "Saving rewind state");
 			Save();
 		}
 
@@ -334,7 +333,10 @@ namespace SaveState
 	static std::string saveStateInitialGitVersion = "";
 
 	// TODO: Should this be configurable?
-	static const int SCREENSHOT_FAILURE_RETRIES = 15;
+	// Should only fail if the game hasn't created a framebuffer yet. Some games play video without a framebuffer,
+	// we should probably just read back memory then instead (GTA for example). But this is a really unimportant edge case,
+	// when in-game it's just not an issue.
+	static const int SCREENSHOT_FAILURE_RETRIES = 6;
 	static StateRingbuffer rewindStates;
 
 	void SaveStart::DoState(PointerWrap &p)
@@ -404,6 +406,9 @@ namespace SaveState
 
 	void Enqueue(const SaveState::Operation &op)
 	{
+		if (!NetworkAllowSaveState()) {
+			return;
+		}
 		if (Achievements::HardcoreModeActive()) {
 			if (g_Config.bAchievementsSaveStateInHardcoreMode && ((op.type == SaveState::SAVESTATE_SAVE) || (op.type == SAVESTATE_SAVE_SCREENSHOT))) {
 				// We allow saving in hardcore mode if this setting is on.
@@ -419,44 +424,49 @@ namespace SaveState
 		// Don't actually run it until next frame.
 		// It's possible there might be a duplicate but it won't hurt us.
 		needsProcess = true;
-		Core_UpdateSingleStep();
 	}
 
-	void Load(const Path &filename, int slot, Callback callback, void *cbUserData)
-	{
+	void Load(const Path &filename, int slot, Callback callback) {
+		if (!NetworkAllowSaveState()) {
+			return;
+		}
+
 		rewindStates.NotifyState();
 		if (coreState == CoreState::CORE_RUNTIME_ERROR)
-			Core_EnableStepping(true, "savestate.load", 0);
-		Enqueue(Operation(SAVESTATE_LOAD, filename, slot, callback, cbUserData));
+			Core_Break(BreakReason::SavestateLoad, 0);
+		Enqueue(Operation(SAVESTATE_LOAD, filename, slot, callback));
 	}
 
-	void Save(const Path &filename, int slot, Callback callback, void *cbUserData)
-	{
+	void Save(const Path &filename, int slot, Callback callback) {
+		if (!NetworkAllowSaveState()) {
+			return;
+		}
+
 		rewindStates.NotifyState();
 		if (coreState == CoreState::CORE_RUNTIME_ERROR)
-			Core_EnableStepping(true, "savestate.save", 0);
-		Enqueue(Operation(SAVESTATE_SAVE, filename, slot, callback, cbUserData));
+			Core_Break(BreakReason::SavestateSave, 0);
+		Enqueue(Operation(SAVESTATE_SAVE, filename, slot, callback));
 	}
 
-	void Verify(Callback callback, void *cbUserData)
-	{
-		Enqueue(Operation(SAVESTATE_VERIFY, Path(), -1, callback, cbUserData));
+	void Verify(Callback callback) {
+		Enqueue(Operation(SAVESTATE_VERIFY, Path(), -1, callback));
 	}
 
-	void Rewind(Callback callback, void *cbUserData)
-	{
+	void Rewind(Callback callback) {
+		if (g_netInited) {
+			return;
+		}
 		if (coreState == CoreState::CORE_RUNTIME_ERROR)
-			Core_EnableStepping(true, "savestate.rewind", 0);
-		Enqueue(Operation(SAVESTATE_REWIND, Path(), -1, callback, cbUserData));
+			Core_Break(BreakReason::SavestateRewind, 0);
+		Enqueue(Operation(SAVESTATE_REWIND, Path(), -1, callback));
 	}
 
-	void SaveScreenshot(const Path &filename, Callback callback, void *cbUserData)
-	{
-		Enqueue(Operation(SAVESTATE_SAVE_SCREENSHOT, filename, -1, callback, cbUserData));
+	static void SaveScreenshot(const Path &filename) {
+		screenshotFailures = 0;
+		Enqueue(Operation(SAVESTATE_SAVE_SCREENSHOT, filename, -1, nullptr));
 	}
 
-	bool CanRewind()
-	{
+	bool CanRewind() {
 		return !rewindStates.Empty();
 	}
 
@@ -524,6 +534,7 @@ namespace SaveState
 		std::string discId = g_paramSFO.GetValueString("DISC_ID");
 		std::string discVer = g_paramSFO.GetValueString("DISC_VERSION");
 		if (discId.empty()) {
+			// Should never happen.
 			discId = g_paramSFO.GenerateFakeID(Path());
 			discVer = "1.00";
 		}
@@ -573,73 +584,83 @@ namespace SaveState
 		}
 	}
 
-	void LoadSlot(const Path &gameFilename, int slot, Callback callback, void *cbUserData)
+	void LoadSlot(const Path &gameFilename, int slot, Callback callback)
 	{
+		if (!NetworkAllowSaveState()) {
+			return;
+		}
+
 		Path fn = GenerateSaveSlotFilename(gameFilename, slot, STATE_EXTENSION);
 		if (!fn.empty()) {
 			// This add only 1 extra state, should we just always enable it?
 			if (g_Config.bEnableStateUndo) {
 				Path backup = GetSysDirectory(DIRECTORY_SAVESTATE) / LOAD_UNDO_NAME;
 				
-				auto saveCallback = [=](Status status, std::string_view message, void *data) {
+				auto saveCallback = [=](Status status, std::string_view message) {
 					if (status != Status::FAILURE) {
 						DeleteIfExists(backup);
 						File::Rename(backup.WithExtraExtension(".tmp"), backup);
 						g_Config.sStateLoadUndoGame = GenerateFullDiscId(gameFilename);
 						g_Config.Save("Saving config for savestate last load undo");
 					} else {
-						ERROR_LOG(SAVESTATE, "Saving load undo state failed: %.*s", (int)message.size(), message.data());
+						ERROR_LOG(Log::SaveState, "Saving load undo state failed: %.*s", (int)message.size(), message.data());
 					}
-					Load(fn, slot, callback, cbUserData);
+					Load(fn, slot, callback);
 				};
 
 				if (!backup.empty()) {
-					Save(backup.WithExtraExtension(".tmp"), LOAD_UNDO_SLOT, saveCallback, cbUserData);
+					Save(backup.WithExtraExtension(".tmp"), LOAD_UNDO_SLOT, saveCallback);
 				} else {
-					ERROR_LOG(SAVESTATE, "Saving load undo state failed. Error in the file system.");
-					Load(fn, slot, callback, cbUserData);
+					ERROR_LOG(Log::SaveState, "Saving load undo state failed. Error in the file system.");
+					Load(fn, slot, callback);
 				}
 			} else {
-				Load(fn, slot, callback, cbUserData);
+				Load(fn, slot, callback);
 			}
 		} else {
 			if (callback) {
 				auto sy = GetI18NCategory(I18NCat::SYSTEM);
-				callback(Status::FAILURE, sy->T("Failed to load state. Error in the file system."), cbUserData);
+				callback(Status::FAILURE, sy->T("Failed to load state. Error in the file system."));
 			}
 		}
 	}
 
-	bool UndoLoad(const Path &gameFilename, Callback callback, void *cbUserData)
-	{
+	bool UndoLoad(const Path &gameFilename, Callback callback) {
+		if (!NetworkAllowSaveState()) {
+			return false;
+		}
+
 		if (g_Config.sStateLoadUndoGame != GenerateFullDiscId(gameFilename)) {
 			if (callback) {
 				auto sy = GetI18NCategory(I18NCat::SYSTEM);
-				callback(Status::FAILURE, sy->T("Error: load undo state is from a different game"), cbUserData);
+				callback(Status::FAILURE, sy->T("Error: load undo state is from a different game"));
 			}
 			return false;
 		}
 
 		Path fn = GetSysDirectory(DIRECTORY_SAVESTATE) / LOAD_UNDO_NAME;
 		if (!fn.empty()) {
-			Load(fn, LOAD_UNDO_SLOT, callback, cbUserData);
+			Load(fn, LOAD_UNDO_SLOT, callback);
 			return true;
 		} else {
 			if (callback) {
 				auto sy = GetI18NCategory(I18NCat::SYSTEM);
-				callback(Status::FAILURE, sy->T("Failed to load state for load undo. Error in the file system."), cbUserData);
+				callback(Status::FAILURE, sy->T("Failed to load state for load undo. Error in the file system."));
 			}
 			return false;
 		}
 	}
 
-	void SaveSlot(const Path &gameFilename, int slot, Callback callback, void *cbUserData)
-	{
+	void SaveSlot(const Path &gameFilename, int slot, Callback callback) {
+		if (!NetworkAllowSaveState()) {
+			return;
+		}
+
 		Path fn = GenerateSaveSlotFilename(gameFilename, slot, STATE_EXTENSION);
 		Path fnUndo = GenerateSaveSlotFilename(gameFilename, slot, UNDO_STATE_EXTENSION);
 		if (!fn.empty()) {
 			Path shot = GenerateSaveSlotFilename(gameFilename, slot, SCREENSHOT_EXTENSION);
-			auto renameCallback = [=](Status status, std::string_view message, void *data) {
+			auto renameCallback = [=](Status status, std::string_view message) {
 				if (status != Status::FAILURE) {
 					if (g_Config.bEnableStateUndo) {
 						DeleteIfExists(fnUndo);
@@ -653,7 +674,7 @@ namespace SaveState
 					File::Rename(fn.WithExtraExtension(".tmp"), fn);
 				}
 				if (callback) {
-					callback(status, message, data);
+					callback(status, message);
 				}
 			};
 			// Let's also create a screenshot.
@@ -662,17 +683,21 @@ namespace SaveState
 				DeleteIfExists(shotUndo);
 				RenameIfExists(shot, shotUndo);
 			}
-			SaveScreenshot(shot, Callback(), 0);
-			Save(fn.WithExtraExtension(".tmp"), slot, renameCallback, cbUserData);
+			SaveScreenshot(shot);
+			Save(fn.WithExtraExtension(".tmp"), slot, renameCallback);
 		} else {
 			if (callback) {
 				auto sy = GetI18NCategory(I18NCat::SYSTEM);
-				callback(Status::FAILURE, sy->T("Failed to save state. Error in the file system."), cbUserData);
+				callback(Status::FAILURE, sy->T("Failed to save state. Error in the file system."));
 			}
 		}
 	}
 
-	bool UndoSaveSlot(const Path &gameFilename, int slot) {		
+	bool UndoSaveSlot(const Path &gameFilename, int slot) {
+		if (!NetworkAllowSaveState()) {
+			return false;
+		}
+
 		Path fnUndo = GenerateSaveSlotFilename(gameFilename, slot, UNDO_STATE_EXTENSION);
 
 		// Do nothing if there's no undo.
@@ -689,15 +714,28 @@ namespace SaveState
 		return false;
 	}
 
+	void DeleteSlot(const Path &gameFilename, int slot) {
+		Path fn = GenerateSaveSlotFilename(gameFilename, slot, STATE_EXTENSION);
+		Path shot = GenerateSaveSlotFilename(gameFilename, slot, SCREENSHOT_EXTENSION);
+
+		if (File::Exists(fn)) {
+			DeleteIfExists(fn);
+			DeleteIfExists(shot);
+		}
+	}
 
 	bool UndoLastSave(const Path &gameFilename) {
+		if (!NetworkAllowSaveState()) {
+			return false;
+		}
+
 		if (g_Config.sStateUndoLastSaveGame != GenerateFullDiscId(gameFilename))
 			return false;
 
 		return UndoSaveSlot(gameFilename, g_Config.iStateUndoLastSaveSlot);
 	}
 
-	bool HasSaveInSlot(const Path &gameFilename, int slot)
+	bool HasSaveInSlot(const Path &gameFilename, int slot) 
 	{
 		Path fn = GenerateSaveSlotFilename(gameFilename, slot, STATE_EXTENSION);
 		return File::Exists(fn);
@@ -802,26 +840,24 @@ namespace SaveState
 
 	std::string GetSlotDateAsString(const Path &gameFilename, int slot) {
 		Path fn = GenerateSaveSlotFilename(gameFilename, slot, STATE_EXTENSION);
-		if (File::Exists(fn)) {
-			tm time;
-			if (File::GetModifTime(fn, time)) {
-				char buf[256];
-				// TODO: Use local time format? Americans and some others might not like ISO standard :)
-				switch (g_Config.iDateFormat) {
-				case PSP_SYSTEMPARAM_DATE_FORMAT_YYYYMMDD:
-					strftime(buf, sizeof(buf), "%Y-%m-%d %H:%M:%S", &time);
-					break;
-				case PSP_SYSTEMPARAM_DATE_FORMAT_MMDDYYYY:
-					strftime(buf, sizeof(buf), "%m-%d-%Y %H:%M:%S", &time);
-					break;
-				case PSP_SYSTEMPARAM_DATE_FORMAT_DDMMYYYY:
-					strftime(buf, sizeof(buf), "%d-%m-%Y %H:%M:%S", &time);
-					break;
-				default: // Should never happen
-					return "";
-				}
-				return std::string(buf);
+		tm time;
+		if (File::GetModifTime(fn, time)) {
+			char buf[256];
+			// TODO: Use local time format? Americans and some others might not like ISO standard :)
+			switch (g_Config.iDateFormat) {
+			case PSP_SYSTEMPARAM_DATE_FORMAT_YYYYMMDD:
+				strftime(buf, sizeof(buf), "%Y-%m-%d %H:%M:%S", &time);
+				break;
+			case PSP_SYSTEMPARAM_DATE_FORMAT_MMDDYYYY:
+				strftime(buf, sizeof(buf), "%m-%d-%Y %H:%M:%S", &time);
+				break;
+			case PSP_SYSTEMPARAM_DATE_FORMAT_DDMMYYYY:
+				strftime(buf, sizeof(buf), "%d-%m-%Y %H:%M:%S", &time);
+				break;
+			default: // Should never happen
+				return "";
 			}
+			return std::string(buf);
 		}
 		return "";
 	}
@@ -835,20 +871,21 @@ namespace SaveState
 		return copy;
 	}
 
-	bool HandleLoadFailure()
+	bool HandleLoadFailure(bool wasRewinding)
 	{
-		WARN_LOG(SAVESTATE, "HandleLoadFailure - trying a rewind state.");
+		if (wasRewinding) {
+			WARN_LOG(Log::SaveState, "HandleLoadFailure - trying a rewind state.");
+			// Okay, first, let's give the next rewind state a shot - maybe we can at least not reset entirely.
+			// Actually, this seems like a bad thing - some systems can end up in bad states, like sceFont :(
+			CChunkFileReader::Error result;
+			do {
+				std::string errorString;
+				result = rewindStates.Restore(&errorString);
+			} while (result == CChunkFileReader::ERROR_BROKEN_STATE);
 
-		// Okay, first, let's give the rewind state a shot - maybe we can at least not reset entirely.
-		// Even if this was a rewind, maybe we can still load a previous one.
-		CChunkFileReader::Error result;
-		do {
-			std::string errorString;
-			result = rewindStates.Restore(&errorString);
-		} while (result == CChunkFileReader::ERROR_BROKEN_STATE);
-
-		if (result == CChunkFileReader::ERROR_NONE) {
-			return true;
+			if (result == CChunkFileReader::ERROR_NONE) {
+				return true;
+			}
 		}
 
 		// We tried, our only remaining option is to reset the game.
@@ -912,29 +949,28 @@ namespace SaveState
 		return Status::SUCCESS;
 	}
 
-	void Process()
-	{
+	// NOTE: This can cause ending of the current renderpass, due to the readback needed for the screenshot.
+	bool Process() {
 		rewindStates.Process();
 
 		if (!needsProcess)
-			return;
+			return false;
 		needsProcess = false;
 
 		if (!__KernelIsRunning())
 		{
-			ERROR_LOG(SAVESTATE, "Savestate failure: Unable to load without kernel, this should never happen.");
-			return;
+			ERROR_LOG(Log::SaveState, "Savestate failure: Unable to load without kernel, this should never happen.");
+			return false;
 		}
 
 		std::vector<Operation> operations = Flush();
 		SaveStart state;
 
-		for (size_t i = 0, n = operations.size(); i < n; ++i)
-		{
-			Operation &op = operations[i];
+		bool readbackImage = false;
+
+		for (const auto &op : operations) {
 			CChunkFileReader::Error result;
 			Status callbackResult;
-			bool tempResult;
 			std::string callbackMessage;
 			std::string title;
 
@@ -948,7 +984,7 @@ namespace SaveState
 			switch (op.type)
 			{
 			case SAVESTATE_LOAD:
-				INFO_LOG(SAVESTATE, "Loading state from '%s'", op.filename.c_str());
+				INFO_LOG(Log::SaveState, "Loading state from '%s'", op.filename.c_str());
 				// Use the state's latest version as a guess for saveStateInitialGitVersion.
 				result = CChunkFileReader::Load(op.filename, &saveStateInitialGitVersion, state, &errorString);
 				if (result == CChunkFileReader::ERROR_NONE) {
@@ -971,10 +1007,11 @@ namespace SaveState
 						}
 					}
 #endif
+					g_lastSaveTime = time_now_d();
 				} else if (result == CChunkFileReader::ERROR_BROKEN_STATE) {
-					HandleLoadFailure();
+					HandleLoadFailure(false);
 					callbackMessage = std::string(i18nLoadFailure) + ": " + errorString;
-					ERROR_LOG(SAVESTATE, "Load state failure: %s", errorString.c_str());
+					ERROR_LOG(Log::SaveState, "Load state failure: %s", errorString.c_str());
 					callbackResult = Status::FAILURE;
 				} else {
 					callbackMessage = sc->T(errorString.c_str(), i18nLoadFailure);
@@ -983,12 +1020,12 @@ namespace SaveState
 				break;
 
 			case SAVESTATE_SAVE:
-				INFO_LOG(SAVESTATE, "Saving state to %s", op.filename.c_str());
+				INFO_LOG(Log::SaveState, "Saving state to '%s'", op.filename.c_str());
 				title = g_paramSFO.GetValueString("TITLE");
 				if (title.empty()) {
 					// Homebrew title
 					title = PSP_CoreParameter().fileToStart.ToVisualString();
-					std::size_t lslash = title.find_last_of("/");
+					std::size_t lslash = title.find_last_of('/');
 					title = title.substr(lslash + 1);
 				}
 				result = CChunkFileReader::Save(op.filename, title, PPSSPP_GIT_VERSION, state);
@@ -1006,10 +1043,11 @@ namespace SaveState
 						}
 					}
 #endif
+					g_lastSaveTime = time_now_d();
 				} else if (result == CChunkFileReader::ERROR_BROKEN_STATE) {
 					// TODO: What else might we want to do here? This should be very unusual.
 					callbackMessage = i18nSaveFailure;
-					ERROR_LOG(SAVESTATE, "Save state failure");
+					ERROR_LOG(Log::SaveState, "Save state failure");
 					callbackResult = Status::FAILURE;
 				} else {
 					callbackMessage = i18nSaveFailure;
@@ -1018,17 +1056,19 @@ namespace SaveState
 				break;
 
 			case SAVESTATE_VERIFY:
-				tempResult = CChunkFileReader::Verify(state) == CChunkFileReader::ERROR_NONE;
+			{
+				int tempResult = CChunkFileReader::Verify(state) == CChunkFileReader::ERROR_NONE;
 				callbackResult = tempResult ? Status::SUCCESS : Status::FAILURE;
 				if (tempResult) {
-					INFO_LOG(SAVESTATE, "Verified save state system");
+					INFO_LOG(Log::SaveState, "Verified save state system");
 				} else {
-					ERROR_LOG(SAVESTATE, "Save state system verification failed");
+					ERROR_LOG(Log::SaveState, "Save state system verification failed");
 				}
 				break;
+			}
 
 			case SAVESTATE_REWIND:
-				INFO_LOG(SAVESTATE, "Rewinding to recent savestate snapshot");
+				INFO_LOG(Log::SaveState, "Rewinding to recent savestate snapshot");
 				result = rewindStates.Restore(&errorString);
 				if (result == CChunkFileReader::ERROR_NONE) {
 					callbackMessage = sc->T("Loaded State");
@@ -1037,7 +1077,7 @@ namespace SaveState
 					Core_ResetException();
 				} else if (result == CChunkFileReader::ERROR_BROKEN_STATE) {
 					// Cripes.  Good news is, we might have more.  Let's try those too, better than a reset.
-					if (HandleLoadFailure()) {
+					if (HandleLoadFailure(true)) {
 						// Well, we did rewind, even if too much...
 						callbackMessage = sc->T("Loaded State");
 						callbackResult = Status::SUCCESS;
@@ -1055,33 +1095,54 @@ namespace SaveState
 
 			case SAVESTATE_SAVE_SCREENSHOT:
 			{
+				_dbg_assert_(!op.callback);
+
 				int maxResMultiplier = 2;
-				tempResult = TakeGameScreenshot(op.filename, ScreenshotFormat::JPG, SCREENSHOT_DISPLAY, nullptr, nullptr, maxResMultiplier);
-				callbackResult = tempResult ? Status::SUCCESS : Status::FAILURE;
-				if (!tempResult) {
-					ERROR_LOG(SAVESTATE, "Failed to take a screenshot for the savestate! %s", op.filename.c_str());
-					if (screenshotFailures++ < SCREENSHOT_FAILURE_RETRIES) {
-						// Requeue for next frame.
-						SaveScreenshot(op.filename, op.callback, op.cbUserData);
+				ScreenshotResult tempResult = TakeGameScreenshot(nullptr, op.filename, ScreenshotFormat::JPG, SCREENSHOT_DISPLAY, maxResMultiplier, [](bool success) {
+					if (success) {
+						screenshotFailures = 0;
 					}
-				} else {
-					screenshotFailures = 0;
+				});
+				
+				switch (tempResult) {
+				case ScreenshotResult::ScreenshotNotPossible:
+					// Try again soon, for a short while.
+					callbackResult = Status::FAILURE;
+					WARN_LOG(Log::SaveState, "Failed to take a screenshot for the savestate! (%s) The savestate will lack an icon.", op.filename.c_str());
+					if (coreState != CORE_STEPPING_CPU && screenshotFailures++ < SCREENSHOT_FAILURE_RETRIES) {
+						// Requeue for next frame (if we were stepping, no point, will just spam errors quickly).
+						SaveScreenshot(op.filename);
+					}
+					break;
+				case ScreenshotResult::DelayedResult:
+				case ScreenshotResult::Success:
+					// We might not know if the file write succeeded yet though.
+					callbackResult = Status::SUCCESS;
+					readbackImage = true;
+					break;
+				case ScreenshotResult::FailedToWriteFile:
+					// Can't reach here when we pass in a callback to TakeGameScreenshot.
+					callbackResult = Status::SUCCESS;
+					break;
 				}
 				break;
 			}
 			default:
-				ERROR_LOG(SAVESTATE, "Savestate failure: unknown operation type %d", op.type);
+				ERROR_LOG(Log::SaveState, "Savestate failure: unknown operation type %d", op.type);
 				callbackResult = Status::FAILURE;
 				break;
 			}
 
-			if (op.callback)
-				op.callback(callbackResult, callbackMessage, op.cbUserData);
+			if (op.callback) {
+				op.callback(callbackResult, callbackMessage);
+			}
 		}
 		if (operations.size()) {
 			// Avoid triggering frame skipping due to slowdown
 			__DisplaySetWasPaused();
 		}
+
+		return readbackImage;
 	}
 
 	void NotifySaveData() {
@@ -1089,20 +1150,12 @@ namespace SaveState
 		lastSaveDataGeneration = saveDataGeneration;
 	}
 
-	void Cleanup() {
+	bool PollRestartNeeded() {
 		if (needsRestart) {
-			PSP_Shutdown();
-			std::string resetError;
-			if (!PSP_Init(PSP_CoreParameter(), &resetError)) {
-				ERROR_LOG(BOOT, "Error resetting: %s", resetError.c_str());
-				// TODO: This probably doesn't clean up well enough.
-				Core_Stop();
-				return;
-			}
-			System_Notify(SystemNotification::BOOT_DONE);
-			System_Notify(SystemNotification::DISASSEMBLY);
 			needsRestart = false;
+			return true;
 		}
+		return false;
 	}
 
 	void Init()
@@ -1118,6 +1171,8 @@ namespace SaveState
 		saveDataGeneration = 0;
 		lastSaveDataGeneration = 0;
 		saveStateInitialGitVersion.clear();
+
+		g_lastSaveTime = time_now_d();
 	}
 
 	void Shutdown()
@@ -1125,4 +1180,12 @@ namespace SaveState
 		std::lock_guard<std::mutex> guard(mutex);
 		rewindStates.Clear();
 	}
+
+	double SecondsSinceLastSavestate() {
+		if (g_lastSaveTime < 0) {
+			return -1.0;
+		} else {
+			return time_now_d() - g_lastSaveTime;
+		}
+	}
 }
