@@ -15,20 +15,16 @@
 // Official git repository and contact information can be found at
 // https://github.com/hrydgard/ppsspp and http://www.ppsspp.org/.
 
-#include <cstdio>
-#include <cstdlib>
-#include <locale.h>
-
 #include "Common/StringUtils.h"
 #include "Common/GPU/OpenGL/GLFeatures.h"
 #include "Common/GPU/ShaderWriter.h"
 #include "Common/GPU/thin3d.h"
 #include "Core/Config.h"
+#include "Core/System.h"
 #include "GPU/ge_constants.h"
 #include "GPU/GPUState.h"
 #include "GPU/Common/ShaderId.h"
 #include "GPU/Common/ShaderUniforms.h"
-#include "GPU/Common/VertexDecoderCommon.h"
 #include "GPU/Common/VertexShaderGenerator.h"
 #include "GPU/Vulkan/DrawEngineVulkan.h"
 
@@ -151,9 +147,6 @@ bool GenerateVertexShader(const VShaderID &id, char *buffer, const ShaderLanguag
 		if (gl_extensions.ARB_cull_distance && id.Bit(VS_BIT_VERTEX_RANGE_CULLING)) {
 			extensions.push_back("#extension GL_ARB_cull_distance : enable");
 		}
-		if (gstate_c.Use(GPU_USE_VIRTUAL_REALITY) && gstate_c.Use(GPU_USE_SINGLE_PASS_STEREO)) {
-			extensions.push_back("#extension GL_OVR_multiview2 : enable\nlayout(num_views=2) in;");
-		}
 	}
 
 	bool useSimpleStereo = id.Bit(VS_BIT_SIMPLE_STEREO);
@@ -210,7 +203,7 @@ bool GenerateVertexShader(const VShaderID &id, char *buffer, const ShaderLanguag
 			*errorString = "Invalid flags - tess requires normal.";
 			return false;
 		}
-		if (compat.shaderLanguage == HLSL_D3D9 || compat.texelFetch == nullptr) {
+		if (compat.texelFetch == nullptr) {
 			*errorString = "Tess not supported on this shader language version";
 			return false;
 		}
@@ -295,83 +288,11 @@ bool GenerateVertexShader(const VShaderID &id, char *buffer, const ShaderLanguag
 		WRITE(p, "layout (location = 3) out highp float v_fogdepth;\n");
 
 		WRITE(p, "invariant gl_Position;\n");
-	} else if (compat.shaderLanguage == HLSL_D3D11 || compat.shaderLanguage == HLSL_D3D9) {
+	} else if (compat.shaderLanguage == HLSL_D3D11) {
 		// Note: These two share some code after this hellishly large if/else.
-		if (compat.shaderLanguage == HLSL_D3D11) {
-			WRITE(p, "cbuffer base : register(b0) {\n%s};\n", ub_baseStr);
-			WRITE(p, "cbuffer lights: register(b1) {\n%s};\n", ub_vs_lightsStr);
-			WRITE(p, "cbuffer bones : register(b2) {\n%s};\n", ub_vs_bonesStr);
-		} else {
-			WRITE(p, "#pragma warning( disable : 3571 )\n");
-			if (isModeThrough) {
-				WRITE(p, "mat4 u_proj_through : register(c%i);\n", CONST_VS_PROJ_THROUGH);
-			} else if (useHWTransform) {
-				WRITE(p, "mat4 u_proj : register(c%i);\n", CONST_VS_PROJ);
-			} else {
-				WRITE(p, "float u_rotation : register(c%i);\n", CONST_VS_ROTATION);
-			}
-
-			if (useHWTransform)
-				WRITE(p, "vec2 u_fogcoef : register(c%i);\n", CONST_VS_FOGCOEF);
-			if (useHWTransform || !hasColor)
-				WRITE(p, "vec4 u_matambientalpha : register(c%i);\n", CONST_VS_MATAMBIENTALPHA);  // matambient + matalpha
-
-			if (useHWTransform) {
-				WRITE(p, "mat3x4 u_world : register(c%i);\n", CONST_VS_WORLD);
-				WRITE(p, "mat3x4 u_view : register(c%i);\n", CONST_VS_VIEW);
-				if (doTextureTransform)
-					WRITE(p, "mat3x4 u_texmtx : register(c%i);\n", CONST_VS_TEXMTX);
-				if (enableBones) {
-#ifdef USE_BONE_ARRAY
-					WRITE(p, "mat3x4 u_bone[%i] : register(c%i);\n", numBones, CONST_VS_BONE0);
-#else
-					for (int i = 0; i < numBoneWeights; i++) {
-						WRITE(p, "mat3x4 u_bone%i : register(c%i);\n", i, CONST_VS_BONE0 + i * 3);
-					}
-#endif
-				}
-				WRITE(p, "vec4 u_uvscaleoffset : register(c%i);\n", CONST_VS_UVSCALEOFFSET);
-				// No need for light ubershader support here, D3D9 doesn't do it.
-				for (int i = 0; i < 4; i++) {
-					if (doLight[i] != LIGHT_OFF) {
-						// This is needed for shade mapping
-						WRITE(p, "vec3 u_lightpos%i : register(c%i);\n", i, CONST_VS_LIGHTPOS + i);
-					}
-					if (doLight[i] == LIGHT_FULL) {
-						GELightType type = static_cast<GELightType>(id.Bits(VS_BIT_LIGHT0_TYPE + 4 * i, 2));
-						GELightComputation comp = static_cast<GELightComputation>(id.Bits(VS_BIT_LIGHT0_COMP + 4 * i, 2));
-
-						if (type != GE_LIGHTTYPE_DIRECTIONAL)
-							WRITE(p, "vec3 u_lightatt%i : register(c%i);\n", i, CONST_VS_LIGHTATT + i);
-
-						if (type == GE_LIGHTTYPE_SPOT || type == GE_LIGHTTYPE_UNKNOWN) {
-							WRITE(p, "vec3 u_lightdir%i : register(c%i);\n", i, CONST_VS_LIGHTDIR + i);
-							WRITE(p, "vec4 u_lightangle_spotCoef%i : register(c%i);\n", i, CONST_VS_LIGHTANGLE_SPOTCOEF + i);
-						}
-						WRITE(p, "vec3 u_lightambient%i : register(c%i);\n", i, CONST_VS_LIGHTAMBIENT + i);
-						WRITE(p, "vec3 u_lightdiffuse%i : register(c%i);\n", i, CONST_VS_LIGHTDIFFUSE + i);
-
-						if (comp == GE_LIGHTCOMP_BOTH) {
-							WRITE(p, "vec3 u_lightspecular%i : register(c%i);\n", i, CONST_VS_LIGHTSPECULAR + i);
-						}
-					}
-				}
-				if (enableLighting) {
-					WRITE(p, "vec4 u_ambient : register(c%i);\n", CONST_VS_AMBIENT);
-					if ((matUpdate & 2) == 0 || !hasColor)
-						WRITE(p, "vec3 u_matdiffuse : register(c%i);\n", CONST_VS_MATDIFFUSE);
-					// if ((matUpdate & 4) == 0)
-					WRITE(p, "vec4 u_matspecular : register(c%i);\n", CONST_VS_MATSPECULAR);  // Specular coef is contained in alpha
-					WRITE(p, "vec3 u_matemissive : register(c%i);\n", CONST_VS_MATEMISSIVE);
-				}
-			}
-
-			if (!isModeThrough) {
-				WRITE(p, "vec4 u_depthRange : register(c%i);\n", CONST_VS_DEPTHRANGE);
-				WRITE(p, "vec4 u_cullRangeMin : register(c%i);\n", CONST_VS_CULLRANGEMIN);
-				WRITE(p, "vec4 u_cullRangeMax : register(c%i);\n", CONST_VS_CULLRANGEMAX);
-			}
-		}
+		WRITE(p, "cbuffer base : register(b0) {\n%s};\n", ub_baseStr);
+		WRITE(p, "cbuffer lights: register(b1) {\n%s};\n", ub_vs_lightsStr);
+		WRITE(p, "cbuffer bones : register(b2) {\n%s};\n", ub_vs_bonesStr);
 
 		// And the "varyings".
 		if (useHWTransform) {
@@ -424,9 +345,7 @@ bool GenerateVertexShader(const VShaderID &id, char *buffer, const ShaderLanguag
 		}
 
 		WRITE(p, "  float v_fogdepth : TEXCOORD1;\n");
-		if (compat.shaderLanguage == HLSL_D3D9) {
-			WRITE(p, "  vec4 gl_Position   : POSITION;\n");
-		} else {
+		{
 			WRITE(p, "  vec4 gl_Position   : SV_Position;\n");
 			bool clipRange = vertexRangeCulling && gstate_c.Use(GPU_USE_CLIP_DISTANCE);
 			if (clipClampedDepth && clipRange) {
@@ -494,11 +413,7 @@ bool GenerateVertexShader(const VShaderID &id, char *buffer, const ShaderLanguag
 			*uniformMask |= DIRTY_PROJTHROUGHMATRIX;
 		} else if (useHWTransform) {
 			if (gstate_c.Use(GPU_USE_VIRTUAL_REALITY)) {
-				if (gstate_c.Use(GPU_USE_SINGLE_PASS_STEREO)) {
-					WRITE(p, "layout(shared) uniform ProjectionMatrix { uniform mat4 u_proj_lens[2]; };\n");
-				} else {
-					WRITE(p, "uniform mat4 u_proj_lens;\n");
-				}
+				WRITE(p, "uniform mat4 u_proj_lens;\n");
 			}
 			WRITE(p, "uniform mat4 u_proj;\n");
 			*uniformMask |= DIRTY_PROJMATRIX;
@@ -508,11 +423,7 @@ bool GenerateVertexShader(const VShaderID &id, char *buffer, const ShaderLanguag
 			// When transforming by hardware, we need a great deal more uniforms...
 			// TODO: Use 4x3 matrices where possible. Though probably doesn't matter much.
 			WRITE(p, "uniform mat4 u_world;\n");
-			if (gstate_c.Use(GPU_USE_VIRTUAL_REALITY) && gstate_c.Use(GPU_USE_SINGLE_PASS_STEREO)) {
-				WRITE(p, "layout(shared) uniform ViewMatrices { uniform mat4 u_view[2]; };\n");
-			} else {
-				WRITE(p, "uniform mat4 u_view;\n");
-			}
+			WRITE(p, "uniform mat4 u_view;\n");
 			*uniformMask |= DIRTY_WORLDMATRIX | DIRTY_VIEWMATRIX;
 			if (doTextureTransform) {
 				WRITE(p, "uniform mediump mat4 u_texmtx;\n");
@@ -662,8 +573,6 @@ bool GenerateVertexShader(const VShaderID &id, char *buffer, const ShaderLanguag
 			WRITE(p, "};\n");
 			WRITE(p, "StructuredBuffer<TessWeight> tess_weights_u : register(t1);\n");
 			WRITE(p, "StructuredBuffer<TessWeight> tess_weights_v : register(t2);\n");
-		} else if (compat.shaderLanguage == HLSL_D3D9) {
-
 		}
 
 		const char *init[3] = { "0.0, 0.0", "0.0, 0.0, 0.0", "0.0, 0.0, 0.0, 0.0" };
@@ -684,7 +593,7 @@ bool GenerateVertexShader(const VShaderID &id, char *buffer, const ShaderLanguag
 			WRITE(p, "mat4 outerProduct(vec4 u, vec4 v) {\n");
 			WRITE(p, "  return mat4(u * v[0], u * v[1], u * v[2], u * v[3]);\n");
 			WRITE(p, "}\n");
-		} else if (compat.shaderLanguage == HLSL_D3D9 || compat.shaderLanguage == HLSL_D3D11) {
+		} else if (compat.shaderLanguage == HLSL_D3D11) {
 			WRITE(p, "mat4 outerProduct(vec4 u, vec4 v) {\n");
 			WRITE(p, "  return mul((float4x1)v, (float1x4)u);\n");
 			WRITE(p, "}\n");
@@ -698,7 +607,7 @@ bool GenerateVertexShader(const VShaderID &id, char *buffer, const ShaderLanguag
 			WRITE(p, "  vec3 nrm;\n");
 		WRITE(p, "};\n");
 
-		if (compat.shaderLanguage == HLSL_D3D9 || compat.shaderLanguage == HLSL_D3D11) {
+		if (compat.shaderLanguage == HLSL_D3D11) {
 			WRITE(p, "void tessellate(in VS_IN In, out Tess tess) {\n");
 			WRITE(p, "  vec3 position = In.position;\n");
 			WRITE(p, "  vec3 normal = In.normal;\n");
@@ -785,7 +694,7 @@ bool GenerateVertexShader(const VShaderID &id, char *buffer, const ShaderLanguag
 
 	if (ShaderLanguageIsOpenGL(compat.shaderLanguage) || compat.shaderLanguage == GLSL_VULKAN) {
 		WRITE(p, "void main() {\n");
-	} else if (compat.shaderLanguage == HLSL_D3D9 || compat.shaderLanguage == HLSL_D3D11) {
+	} else if (compat.shaderLanguage == HLSL_D3D11) {
 		WRITE(p, "VS_OUT main(VS_IN In) {\n");
 		WRITE(p, "  VS_OUT Out;\n");
 		if (hasTexcoord) {
@@ -866,7 +775,7 @@ bool GenerateVertexShader(const VShaderID &id, char *buffer, const ShaderLanguag
 			if (doBezier || doSpline) {
 				// Hardware tessellation
 				WRITE(p, "  Tess tess;\n");
-				if (compat.shaderLanguage == HLSL_D3D9 || compat.shaderLanguage == HLSL_D3D11) {
+				if (compat.shaderLanguage == HLSL_D3D11) {
 					WRITE(p, "  tessellate(In, tess);\n");
 				} else {
 					WRITE(p, "  tessellate(tess);\n");
@@ -926,24 +835,27 @@ bool GenerateVertexShader(const VShaderID &id, char *buffer, const ShaderLanguag
 			WRITE(p, "  mediump vec3 worldnormal = normalizeOr001(mul(vec4(skinnednormal, 0.0), u_world).xyz);\n");
 		}
 
-		std::string matrixPostfix;
-		if (gstate_c.Use(GPU_USE_VIRTUAL_REALITY) && gstate_c.Use(GPU_USE_SINGLE_PASS_STEREO)) {
-			matrixPostfix = "[gl_ViewID_OVR]";
+		WRITE(p, "  vec4 viewPos = vec4(mul(vec4(worldpos, 1.0), u_view).xyz, 1.0);\n");
+		if (useSimpleStereo) {
+			float ipd = 0.065f;
+			float scale = 1.0f;
+			if (PSP_CoreParameter().compat.vrCompat().UnitsPerMeter > 0) {
+				scale = PSP_CoreParameter().compat.vrCompat().UnitsPerMeter;
+			}
+			WRITE(p, "  viewPos.x += %f * float(gl_ViewIndex * 2 - 1);\n", scale * ipd * 0.5);
 		}
 
-		WRITE(p, "  vec4 viewPos = vec4(mul(vec4(worldpos, 1.0), u_view%s).xyz, 1.0);\n", matrixPostfix.c_str());
-
 		// Final view and projection transforms.
 		if (gstate_c.Use(GPU_ROUND_DEPTH_TO_16BIT)) {
 			if (gstate_c.Use(GPU_USE_VIRTUAL_REALITY)) {
-				WRITE(p, "  vec4 outPos = depthRoundZVP(mul(u_proj_lens%s, viewPos));\n", matrixPostfix.c_str());
+				WRITE(p, "  vec4 outPos = depthRoundZVP(mul(u_proj_lens, viewPos));\n");
 				WRITE(p, "  vec4 orgPos = depthRoundZVP(mul(u_proj, viewPos));\n");
 			} else {
 				WRITE(p, "  vec4 outPos = depthRoundZVP(mul(u_proj, viewPos));\n");
 			}
 		} else {
 			if (gstate_c.Use(GPU_USE_VIRTUAL_REALITY)) {
-				WRITE(p, "  vec4 outPos = mul(u_proj_lens%s, viewPos);\n", matrixPostfix.c_str());
+				WRITE(p, "  vec4 outPos = mul(u_proj_lens, viewPos);\n");
 				WRITE(p, "  vec4 orgPos = mul(u_proj, viewPos);\n");
 			} else {
 				WRITE(p, "  vec4 outPos = mul(u_proj, viewPos);\n");
@@ -1019,6 +931,7 @@ bool GenerateVertexShader(const VShaderID &id, char *buffer, const ShaderLanguag
 			}
 		}
 
+		// NOTE: Can't change this without updating uniform buffer declarations (for D3D11 and VK, the one in ShaderUniforms.h).
 		bool useIndexing = compat.shaderLanguage == HLSL_D3D11 || compat.shaderLanguage == GLSL_VULKAN;
 
 		char iStr[4];
@@ -1069,7 +982,7 @@ bool GenerateVertexShader(const VShaderID &id, char *buffer, const ShaderLanguag
 				p.C("      ldot = u_matspecular.a > 0.0 ? pow(max(ldot, 0.0), u_matspecular.a) : 1.0;\n");
 				p.C("    }\n");
 				p.F("    diffuse = (u_lightdiffuse%s * diffuseColor) * max(ldot, 0.0);\n", iStr);
-				p.C("    if (comp == 0x1u && ldot > 0.0) {\n");  // do specular
+				p.C("    if (comp == 0x1u && ldot >= 0.0) {\n");  // do specular. note - must allow for the >= case, since the u_matspecular.a <= 0.0 case relies on it.
 				p.C("      if (u_matspecular.a > 0.0) {\n");
 				p.C("        ldot = dot(normalize(toLight + vec3(0.0, 0.0, 1.0)), worldnormal);\n");
 				p.C("        ldot = pow(max(ldot, 0.0), u_matspecular.a);\n");
@@ -1132,7 +1045,7 @@ bool GenerateVertexShader(const VShaderID &id, char *buffer, const ShaderLanguag
 				case GE_LIGHTTYPE_UNKNOWN:
 					p.F("  angle = dot(u_lightdir%s, toLight);\n", iStr, iStr);
 					p.F("  if (angle >= u_lightangle_spotCoef%s.x) {\n", iStr);
-					p.F("    lightScale = clamp(1.0 / dot(u_lightatt%s, vec3(1.0, distance, distance*distance)), 0.0, 1.0) * (u_lightangle_spotCoef%s.y <= 0.0 ? 1.0 : pow(angle, u_lightangle_spotCoef%s.y));\n", iStr, iStr, iStr);
+					p.F("    lightScale = clamp(1.0 / dot(u_lightatt%s, vec3(1.0, distance, distance*distance)), 0.0, 1.0) * (u_lightangle_spotCoef%s.y <= 0.0 ? 1.0 : pow(max(angle, 0.0), u_lightangle_spotCoef%s.y));\n", iStr, iStr, iStr);
 					p.C("  } else {\n");
 					p.C("    lightScale = 0.0;\n");
 					p.C("  }\n");
@@ -1313,7 +1226,7 @@ bool GenerateVertexShader(const VShaderID &id, char *buffer, const ShaderLanguag
 		WRITE(p, "  }\n");
 	}
 
-	if (vertexRangeCulling && !gstate_c.Use(GPU_USE_VIRTUAL_REALITY)) {
+	if (vertexRangeCulling) {
 		WRITE(p, "  vec3 projPos = outPos.xyz / outPos.w;\n");
 		WRITE(p, "  float projZ = (projPos.z - u_depthRange.z) * u_depthRange.w;\n");
 
@@ -1367,19 +1280,13 @@ bool GenerateVertexShader(const VShaderID &id, char *buffer, const ShaderLanguag
 		WRITE(p, "  %sgl_Position.y *= u_scaleY;\n", compat.vsOutPrefix);
 	}
 
-	if (useSimpleStereo && useHWTransform) {
-		p.C("  float zFactor = 0.2 * float(gl_ViewIndex * 2 - 1);\n");
-		p.C("  float zFocus = 0.0;\n");
-		p.C("  gl_Position.x += (-gl_Position.z - zFocus) * zFactor;\n");
-	}
-
 	if (needsZWHack) {
 		// See comment in thin3d_vulkan.cpp.
 		WRITE(p, "  if (%sgl_Position.z == %sgl_Position.w) %sgl_Position.z *= 0.999999;\n",
 			compat.vsOutPrefix, compat.vsOutPrefix, compat.vsOutPrefix);
 	}
 
-	if (compat.shaderLanguage == HLSL_D3D11 || compat.shaderLanguage == HLSL_D3D9) {
+	if (compat.shaderLanguage == HLSL_D3D11) {
 		WRITE(p, "  return Out;\n");
 	}
 	WRITE(p, "}\n");
