@@ -1,11 +1,3 @@
-// Debugging notes
-// The crash happens when we try to call vkGetPhysicalDeviceProperties2KHR which seems to be null.
-//
-// Apparently we don't manage to specify the extensions we want. Still something reports that this one
-// is present?
-// Failed to load : vkGetPhysicalDeviceProperties2KHR
-// Failed to load : vkGetPhysicalDeviceFeatures2KHR
-
 #include <cstring>
 #include <cassert>
 #include <vector>
@@ -80,41 +72,79 @@ static VKAPI_ATTR VkResult VKAPI_CALL vkCreateInstance_libretro(const VkInstance
 }
 
 static void add_name_unique(std::vector<const char *> &list, const char *value) {
-	for (const char *name : list)
-		if (!strcmp(value, name))
-			return;
+   for (const char *name : list) {
+      if (!strcmp(value, name))
+         return;
+   }
 
-	list.push_back(value);
+   list.push_back(value);
 }
+
 static VKAPI_ATTR VkResult VKAPI_CALL vkCreateDevice_libretro(VkPhysicalDevice physicalDevice, const VkDeviceCreateInfo *pCreateInfo, const VkAllocationCallbacks *pAllocator, VkDevice *pDevice) {
-	VkDeviceCreateInfo info = *pCreateInfo;
-	std::vector<const char *> EnabledLayerNames(info.ppEnabledLayerNames, info.ppEnabledLayerNames + info.enabledLayerCount);
-	std::vector<const char *> EnabledExtensionNames(info.ppEnabledExtensionNames, info.ppEnabledExtensionNames + info.enabledExtensionCount);
-	VkPhysicalDeviceFeatures EnabledFeatures = *info.pEnabledFeatures;
+   VkDeviceCreateInfo newInfo = *pCreateInfo;
 
-	for (unsigned i = 0; i < vk_init_info.num_required_device_layers; i++)
-		add_name_unique(EnabledLayerNames, vk_init_info.required_device_layers[i]);
+   // Add our custom layers
+   std::vector<const char *> enabledLayerNames(pCreateInfo->ppEnabledLayerNames, pCreateInfo->ppEnabledLayerNames + pCreateInfo->enabledLayerCount);
 
-	for (unsigned i = 0; i < vk_init_info.num_required_device_extensions; i++)
-		add_name_unique(EnabledExtensionNames, vk_init_info.required_device_extensions[i]);
+   for (uint32_t i = 0; i < vk_init_info.num_required_device_layers; i++) {
+      add_name_unique(enabledLayerNames, vk_init_info.required_device_layers[i]);
+   }
 
-	for (unsigned i = 0; i < sizeof(VkPhysicalDeviceFeatures) / sizeof(VkBool32); i++) {
-		if (((VkBool32 *)vk_init_info.required_features)[i])
-			((VkBool32 *)&EnabledFeatures)[i] = VK_TRUE;
-	}
+   newInfo.enabledLayerCount = (uint32_t)enabledLayerNames.size();
+   newInfo.ppEnabledLayerNames = newInfo.enabledLayerCount ? enabledLayerNames.data() : nullptr;
 
-	for (auto extension_name : EnabledExtensionNames) {
-		if (!strcmp(extension_name, VK_KHR_DEDICATED_ALLOCATION_EXTENSION_NAME))
-			DEDICATED_ALLOCATION = true;
-	}
+   // Add our custom extensions
+   std::vector<const char *> enabledExtensionNames(pCreateInfo->ppEnabledExtensionNames, pCreateInfo->ppEnabledExtensionNames + pCreateInfo->enabledExtensionCount);
 
-	info.enabledLayerCount = (uint32_t)EnabledLayerNames.size();
-	info.ppEnabledLayerNames = info.enabledLayerCount ? EnabledLayerNames.data() : nullptr;
-	info.enabledExtensionCount = (uint32_t)EnabledExtensionNames.size();
-	info.ppEnabledExtensionNames = info.enabledExtensionCount ? EnabledExtensionNames.data() : nullptr;
-	info.pEnabledFeatures = &EnabledFeatures;
+   for (uint32_t i = 0; i < vk_init_info.num_required_device_extensions; i++) {
+      add_name_unique(enabledExtensionNames, vk_init_info.required_device_extensions[i]);
+   }
 
-	return vkCreateDevice_org(physicalDevice, &info, pAllocator, pDevice);
+   for (const char *extensionName : enabledExtensionNames) {
+      if (!strcmp(extensionName, VK_KHR_DEDICATED_ALLOCATION_EXTENSION_NAME))
+         DEDICATED_ALLOCATION = true;
+   }
+
+   newInfo.enabledExtensionCount = (uint32_t)enabledExtensionNames.size();
+   newInfo.ppEnabledExtensionNames = newInfo.enabledExtensionCount ? enabledExtensionNames.data() : nullptr;
+
+   // Then check for VkPhysicalDeviceFeatures2 chaining or pEnabledFeatures to enable required features. Note that when both
+   // structs are present Features2 takes precedence. vkCreateDevice parameters don't give us a simple way to detect
+   // VK_KHR_get_physical_device_properties2 usage so we'll always try both paths.
+   std::unordered_map<VkPhysicalDeviceFeatures *, VkPhysicalDeviceFeatures> originalFeaturePointers;
+   VkPhysicalDeviceFeatures placeholderEnabledFeatures{};
+
+   for (const VkBaseOutStructure *next = (const VkBaseOutStructure *)pCreateInfo->pNext; next != nullptr;) {
+      if (next->sType == VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FEATURES_2) {
+         VkPhysicalDeviceFeatures *enabledFeatures = &((VkPhysicalDeviceFeatures2 *)next)->features;
+         originalFeaturePointers.try_emplace(enabledFeatures, *enabledFeatures);
+      }
+
+      next = (const VkBaseOutStructure *)next->pNext;
+   }
+
+   if (newInfo.pEnabledFeatures) {
+      placeholderEnabledFeatures = *newInfo.pEnabledFeatures;
+   }
+
+   newInfo.pEnabledFeatures = &placeholderEnabledFeatures;
+   originalFeaturePointers.try_emplace((VkPhysicalDeviceFeatures *)newInfo.pEnabledFeatures, *newInfo.pEnabledFeatures);
+
+   for (const auto& pair : originalFeaturePointers) {
+      for (uint32_t i = 0; i < sizeof(VkPhysicalDeviceFeatures) / sizeof(VkBool32); i++) {
+         if (((VkBool32 *)vk_init_info.required_features)[i])
+            ((VkBool32 *)pair.first)[i] = VK_TRUE;
+      }
+   }
+
+   VkResult res = vkCreateDevice_org(physicalDevice, &newInfo, pAllocator, pDevice);
+
+   // The above code potentially modifies application memory. Restore it to avoid unexpected side effects.
+   for (const auto& pair : originalFeaturePointers) {
+      *pair.first = pair.second;
+   }
+
+   return res;
 }
 
 static VKAPI_ATTR VkResult VKAPI_CALL vkCreateLibretroSurfaceKHR(VkInstance instance, const void *pCreateInfo, const VkAllocationCallbacks *pAllocator, VkSurfaceKHR *pSurface) {
@@ -125,14 +155,17 @@ static VKAPI_ATTR VkResult VKAPI_CALL vkCreateLibretroSurfaceKHR(VkInstance inst
 VKAPI_ATTR VkResult VKAPI_CALL vkGetPhysicalDeviceSurfaceCapabilitiesKHR_libretro(VkPhysicalDevice physicalDevice, VkSurfaceKHR surface, VkSurfaceCapabilitiesKHR *pSurfaceCapabilities) {
 	VkResult res = vkGetPhysicalDeviceSurfaceCapabilitiesKHR_org(physicalDevice, surface, pSurfaceCapabilities);
 	if (res == VK_SUCCESS) {
-      int w = g_Config.iInternalResolution * 480;
-      int h = g_Config.iInternalResolution * 272;
+		int w = g_Config.iInternalResolution * NATIVEWIDTH;
+		int h = g_Config.iInternalResolution * NATIVEHEIGHT;
 
-      pSurfaceCapabilities->minImageExtent.width = w;
-      pSurfaceCapabilities->minImageExtent.height = h;
-      pSurfaceCapabilities->maxImageExtent.width = w;
-      pSurfaceCapabilities->maxImageExtent.height = h;
-      pSurfaceCapabilities->currentExtent.width = w;
+		if (g_Config.bDisplayCropTo16x9)
+			h -= g_Config.iInternalResolution * 2;
+
+		pSurfaceCapabilities->minImageExtent.width = w;
+		pSurfaceCapabilities->minImageExtent.height = h;
+		pSurfaceCapabilities->maxImageExtent.width = w;
+		pSurfaceCapabilities->maxImageExtent.height = h;
+		pSurfaceCapabilities->currentExtent.width = w;
 		pSurfaceCapabilities->currentExtent.height = h;
 	}
 	return res;
@@ -207,7 +240,7 @@ static VKAPI_ATTR VkResult VKAPI_CALL vkCreateSwapchainKHR_libretro(VkDevice dev
 		chain.images[i].retro_image.create_info.image = chain.images[i].handle;
 		chain.images[i].retro_image.create_info.viewType = VK_IMAGE_VIEW_TYPE_2D;
 		chain.images[i].retro_image.create_info.format = pCreateInfo->imageFormat;
-		chain.images[i].retro_image.create_info.components = { VK_COMPONENT_SWIZZLE_R, VK_COMPONENT_SWIZZLE_G, VK_COMPONENT_SWIZZLE_B, VK_COMPONENT_SWIZZLE_A };
+		chain.images[i].retro_image.create_info.components = { VK_COMPONENT_SWIZZLE_IDENTITY, VK_COMPONENT_SWIZZLE_IDENTITY, VK_COMPONENT_SWIZZLE_IDENTITY, VK_COMPONENT_SWIZZLE_IDENTITY };
 		chain.images[i].retro_image.create_info.subresourceRange.aspectMask = VK_IMAGE_ASPECT_COLOR_BIT;
 		chain.images[i].retro_image.create_info.subresourceRange.layerCount = 1;
 		chain.images[i].retro_image.create_info.subresourceRange.levelCount = 1;
@@ -373,9 +406,9 @@ VKAPI_ATTR PFN_vkVoidFunction VKAPI_CALL vkGetInstanceProcAddr_libretro(VkInstan
 		return (PFN_vkVoidFunction)vkCreateLibretroSurfaceKHR;
 	}
 
-	PFN_vkVoidFunction fptr = vkGetInstanceProcAddr_org(instance, pName);
+	PFN_vkVoidFunction fptr = vk_init_info.get_instance_proc_addr(instance, pName);
    if (!fptr) {
-      ERROR_LOG(G3D, "Failed to load VK instance function: %s", pName);
+      ERROR_LOG(Log::G3D, "Failed to load VK instance function: %s", pName);
       return fptr;
    }
 
@@ -409,9 +442,13 @@ void vk_libretro_init(VkInstance instance, VkPhysicalDevice gpu, VkSurfaceKHR su
 
 	vkGetInstanceProcAddr_org = vkGetInstanceProcAddr;
 	vkGetInstanceProcAddr = vkGetInstanceProcAddr_libretro;
-	vkGetDeviceProcAddr_org = vkGetDeviceProcAddr;
+	vkGetDeviceProcAddr_org = (PFN_vkGetDeviceProcAddr)vkGetInstanceProcAddr(instance, "vkGetDeviceProcAddr");;
 	vkGetDeviceProcAddr = vkGetDeviceProcAddr_libretro;
 	vkCreateInstance = vkCreateInstance_libretro;
+
+	vkEnumerateInstanceVersion = (PFN_vkEnumerateInstanceVersion)vkGetInstanceProcAddr(NULL, "vkEnumerateInstanceVersion");
+	vkEnumerateInstanceExtensionProperties = (PFN_vkEnumerateInstanceExtensionProperties)vkGetInstanceProcAddr(NULL, "vkEnumerateInstanceExtensionProperties");
+	vkEnumerateInstanceLayerProperties = (PFN_vkEnumerateInstanceLayerProperties)vkGetInstanceProcAddr(NULL, "vkEnumerateInstanceLayerProperties");
 }
 
 void vk_libretro_set_hwrender_interface(retro_hw_render_interface *hw_render_interface) {
